﻿unit ImportU;

interface

uses
  Windows, SysUtils, TntSysUtils, StrUtils, Math, DateUtils;

const
  PLUGIN_UNKNOWN = 0;
  PLUGIN_GENERAL = 1;
  PLUGIN_WEBIMPORT = 2;
  PLUGIN_GENIMPORT = 3;
  PLUGIN_WEBPICIMPORT = 4;
  PLUGIN_WEBCOVERIMPORT = 5;
  PLUGIN_WEBPEOPLEIMPORT = 6;
  PLUGIN_EXPORT = 7;

  PLUGIN_RESPONSE_ERROR = 0;
  PLUGIN_RESPONSE_OK = 1;
  PLUGIN_RESPONSE_LIST = 2;
  PLUGIN_RESPONSE_DOWNLOAD = 3;
  PLUGIN_RESPONSE_OKPHOTO = 4;

  WEB_URL_NONE   = -1;
  WEB_URL_SEARCH = 0;
  WEB_URL_DIRECT = 1;

  //TCoverType
  ctFront = 0;
  ctCD    = 1;

  //Plugin Errors
  ERR_PLUG_OK            = 0;
  ERR_PLUG_NEEDCONFIGURE = 1;


type
  TWideArray = array of WideString;
  TIntArray = array of Integer;

  TPluginType = PLUGIN_UNKNOWN..PLUGIN_EXPORT;

  //Web Import
  PSearchResult = ^TSearchResult;
  TSearchResult = record
    Year : Integer;
    Title : PWideChar;
    Title2 : PWideChar;
    URL : PCHAR;
    Preview : PCHAR;
    Browse : PCHAR;
  end;

  TSelectionRec = record
   Year : Integer;
   Title : WideString;
   Title2 : WideString;
   URL : String;
   Preview : String;
   Browse : String;
  end;

  TMovieRec = record
   Title : WideString;
   OrigTitle : WideString;
   Aka : WideString;
   Year : SmallInt;
   Genre : WideString;
   Director : WideString;
   Producer : WideString;
   Scenario : WideString;
   Music : WideString;
   Actors : WideString;
   Country : WideString;
   Studio : WideString;
   MPAA : WideString;
   Rating : Extended;
   RatingName : WideString;
   Tags : WideString;
   Tagline : WideString;
   Description : WideString;
   Duration : Integer;
   Awards : WideString;
   Links : WideString;
   URL : WideString;
   Features : WideString;
   Episodes : WideString;
  end;

  THumanRec = record
    Name : WideString;
    TransName : WideString;
    AltNames : WideString;
    Birthday : WideString;
    Birthplace : WideString;
    DeathDate : WideString;
    Genre : WideString;
    URL : WideString;
    Bio : WideString;
    Films : WideString;
    Awards : WideString;
  end;

  TCoverType = ctFront..ctCD;
  TURLList = array of String;

  TWorkBeginFunc = function : Integer; stdcall;
  TWorkEndFunc = procedure; stdcall;

  TPluginImpGenMovieCallback = function (Movie : PWideChar) : BOOL;
  TPluginImpGenPersonCallback = function (Person : PWideChar) : BOOL;
  TPluginImpGenFilterFunc = function (AppHandle : HWND; const LangID : Integer; CustomFields : PWideChar) : PWideChar; stdcall;
  TPluginImpGenParseFunc = procedure (AppHandle : HWND; const LangID : Integer; FileName : PWideChar; MovieCallback : TPluginImpGenMovieCallback; HumanCallback : TPluginImpGenPersonCallback; CustomFields : PWideChar); stdcall;

  TPluginCanConfigFunc = function : BOOL; stdcall;
  TPluginConfigFunc = function (const LangID : Integer; AppHandle : HWND; CustomFields : PWideChar) : BOOL; stdcall;
  TPluginTypeFunc = function : SHORT; stdcall;
  TPluginNameFunc = function (const LangID : Integer) : PWideChar; stdcall;
  TPluginDescFunc = function (const LangID : Integer) : PWideChar; stdcall;
  TPluginVerFunc = function : PCHAR; stdcall;
  TPluginLangFunc = function : Integer; stdcall;
  TPluginSupVerFunc = function : DWORD; stdcall;
  TPluginCheckVerFunc = function (const V : Integer) : BOOL; stdcall;
  TPluginAboutFunc = function (const LangID : Integer) : PWideChar; stdcall;

  TPluginImpWebSelTitleFunc = function (const LangID : Integer) : PWideChar; stdcall;
  TPluginImpWebRefFunc = function : PCHAR; stdcall;
  TPluginImpWebParseFunc = function (URL : PChar; Data : PWideChar) : SHORT; stdcall;
  TPluginImpWebCountFunc = function : Integer; stdcall;
  TPluginImpWebGetListFunc = function (const N : Integer; out Title, Title2 : PWideChar; out URL, Preview, Browse : PCHAR; out Year : Smallint) : BOOL; stdcall; //PreviewURL is only used for picture preview
  TPluginImpWebLoginFunc = procedure (out ScriptURL : PCHAR; out Params : PCHAR); stdcall;
  TPluginImpWebCodePageFunc = function : Cardinal; stdcall;

  TPluginImpWebURLFunc = function(const isEpisode : Boolean; const CurrentURL : PCHAR; out NewURL : PCHAR) : Integer; stdcall;
  TPluginImpPeopleWebURLFunc = function(const CurrentURL : PCHAR; out NewURL : PCHAR) : Integer; stdcall;
  TPluginImpCoverWebURLFunc = function (const CurrentURL : PCHAR; out NewURL : PCHAR; const ASearchText : PChar) : Integer; stdcall;
  TPluginImpWebGetMovieFunc = function (const N : Integer; var AMovieInfo : PWideChar) : BOOL; stdcall;
  TPluginImpWebGetHumanFunc = function (const N : Integer; var AHumanInfo : PWideChar) : Integer; stdcall;
  TPluginImpWebSetSelFunc = procedure (const AVal : Integer); stdcall;

  TPluginImpWebCoverTypeFunc = function : SHORT; stdcall;

  TExpValueCallback = function (ValueIndex : Integer; IsCustomIndex : Boolean) : PWideChar; //if IsCustomIndex then ValueIndex is a number of custom field
  TExpNextCallback = function : BOOL;

  TPluginExpSettingsFunc = procedure (AppHandle : HWND; const LangID : Integer; CustomFields : PWideChar; out AFilter : PWideChar; out SortField : Integer; out SortDirection : Boolean); stdcall;
  TPluginExportFunc = function (AppHandle : HWND; const LangID : Integer; FileName : PWideChar; const ACount : Integer; const AType : Integer; dbName : PWideChar; ValueCallback : TExpValueCallback; NextCallback : TExpNextCallback) : BOOL; stdcall;
  TPluginExpTypeFunc = function (const AType : Integer) : BOOL; stdcall;

  function TimeStrToSQL(AString : String) : String;
  function DateStrToSQL(AString : String) : String;
  function TimeStrSQLToTime(AString : String) : TDateTime;
  function MinutesToTimeStr(Minutes : Integer; SQL : Boolean = True) : String;
  function MinutesStrToTimeStr(Minutes : String; SQL : Boolean = True) : String;
  function MinutesToTime(Minutes : Integer) : TDateTime;
  function MinutesStrToTime(Minutes : String) : TDateTime;
  function TimeToMinutesStr(Time : TDateTime) : String;
  function StringToIntIgnore(AString : String) : Integer;
  function SizeStringToMBIntIgnore(AString : String) : Integer;
  function FloatToStrSQL(const AValue : Extended) : String;
  function FloatStrToSQL(const AValue : String) : String;
  function FloatStrToFloat(const AValue : String) : Extended;
  function DateToStrSQL(const AValue : TDateTime) : String;
  function TimeToStrSQL(const AValue : TDateTime) : String;
  function SecondsToTimeString(AValue : Integer) : String;
  function TimeStringToSeconds(AValue : String) : Integer;
  function TimeToSeconds(AValue : TDateTime) : Integer;

  function ConvertEncoding(const HTML : String; CP : Cardinal) : WideString;
  function WideStringToMultibyte(const AText : WideString; CP : Cardinal) : String;

  function HTMLToWideString(const HTML : String; DefCP : Cardinal = 0) : WideString;
  function HTMLToText(const HTML : WideString) : WideString; overload;
  function TextToHTML(TheText: String): String;

  function URLDecode(const S: String): String;
  function URLEncode(const S: String; const InQueryString: Boolean = True): String;
  function URLEncodeWide(const S : WideString; CP : Cardinal) : String;

  procedure ClearMovie(var Movie : TMovieRec);
  procedure ClearHuman(var Human : THumanRec);
  function ParseURLs(AText : String; var URLs : TURLList) : Integer;
  function ExplodeCustomFields(AText : PWideChar; var Fields : TWideArray; var Indexes : TIntArray) : Integer; overload;
  function ExplodeString2(AText : WideString; var Items : TWideArray; const Delimiters : WideString = ',/\;|') : Integer;

  function CreateCustomTag(FieldID : Integer; AValue : WideString) : WideString;
  function CreateFieldTag(AField : Integer; AValue : WideString) : WideString;

  function RemoveTags(AText : WideString; doLineBreaks : Boolean = False) : WideString;

  function PosFrom(const SubStr, Str : WideString; FromIndex : Integer) : Integer;
  function LastPos(const SubStr, Str : WideString) : Integer;

implementation

uses XMLGenU;

function PosFrom(const SubStr, Str : WideString; FromIndex : Integer) : Integer;
var
 I, L : Integer;
begin
 Result := 0;

 L := Length(SubStr);
 if (FromIndex < 1) OR (FromIndex > Length(Str) - L) then
  Exit;

 for I := FromIndex to Length(Str) - L do
  if WidecompareText(SubStr, Copy(Str, I, L)) = 0 then begin
   Result := I;
   Break;
  end;
end;

function LastPos(const SubStr, Str : WideString) : Integer;
var
 I, L : Integer;
begin
 Result := 0;

 L := Length(SubStr);

 for I := Length(Str) - L downto 1 do
  if WidecompareText(SubStr, Copy(Str, I, L)) = 0 then begin
   Result := I;
   Break;
  end;
end;

function RemoveTags(AText : WideString; doLineBreaks : Boolean = False) : WideString;
var
 B, E : Integer;
begin
 Result := AText;

 Result := Tnt_WideStringReplace(Result, #13#10, '', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, #13, '', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, #10, '', [rfReplaceAll]);
 if doLineBreaks then begin
  Result := Tnt_WideStringReplace(Result, '<br>', #13#10, [rfReplaceAll]);
  Result := Tnt_WideStringReplace(Result, '<br/>', #13#10, [rfReplaceAll]);
 end;


 B := Pos('<', Result);
 E := Pos('>', Result);

 while (B > 0) AND (B < E) do begin
  Delete(Result, B, E - B + 1);

  B := Pos('<', Result);
  E := Pos('>', Result);
 end;
end;

function CreateFieldTag(AField : Integer; AValue : WideString) : WideString;
var
 V : TWideArray;
 I : Integer;

 D : TDateTime;
begin
 if AValue = '' then begin
  Result := '';
  Exit;
 end;

 if (AField = 8) OR (AField = 9) OR (AField = 10) OR (AField = 21) then
  AValue := FloatStrToSQL(AValue)
 else
 if AField = 18 then begin
  D := TimeStrSQLToTime(AValue);
  if D = 0 then
   AValue := IntToStr(StrToIntDef(AValue, 0) * 60)
  else
   AValue := TimeToMinutesStr(D);
 end else
 if (AField = 16) OR (AField = 43) then
  AValue := DateStrToSQL(AValue)
 else
 if AField = 26 then
  AValue := IntToStr(SizeStringToMBIntIgnore(AValue))
 else
 if (AField = 32) OR (AField = 33) OR (AField = 42) then begin
  SetLength(TrueBoolStrs, 4);
  SetLength(FalseBoolStrs, 3);

  TrueBoolStrs[0] := 'True';
  TrueBoolStrs[1] := '-1';
  TrueBoolStrs[2] := '1';
  TrueBoolStrs[3] := 'Yes';

  FalseBoolStrs[0] := 'False';
  FalseBoolStrs[1] := '0';
  FalseBoolStrs[2] := 'No';

  AValue := BoolToStr(StrToBool(AValue), False);

  SetLength(TrueBoolStrs, 0);
  SetLength(FalseBoolStrs, 0);
 end;

 //Warning these values <> Movie Field constantans (mfMID..mfEpisodes)!!!
 case AField of
  0 : Result := CreateTag(XML_NUMBER, AValue);
  1 : Result := CreateTag(XML_TITLE, AValue);
  2 : Result := CreateTag(XML_ORIGTITLE, AValue);
  3 : Result := CreateTag(XML_AKA, AValue);
  4 : Result := CreateTag(XML_YEAR, AValue);
  5 : Result := CreateTag(XML_MPAA, AValue);
  6 : Result := CreateTag(XML_RELEASE, AValue);
  7 : Result := CreateTag(XML_URL, AValue);
  8 : Result := CreateTag(XML_IMDBRATING, AValue);
  9 : Result := CreateTag(XML_RATING, AValue);
  10 : Result := CreateTag(XML_OTHERRATING, AValue);
  11 : Result := CreateTag(XML_OTHERNAME, AValue);
  12 : Result := CreateTag(XML_LOCATION, AValue);
  13 : Result := CreateTag(XML_TAGLINE, AValue);
  14 : Result := CreateTag(XML_DESCRIPTION, AValue);
  15 : Result := CreateTag(XML_COMMENT, AValue);
  16 : Result := CreateTag(XML_DATE, AValue);
  17 : Result := CreateTag(XML_RIPTYPE, AValue);
  18 : Result := CreateTag(XML_DURATION, AValue);
  19 : Result := CreateTag(XML_TRANSLATION, AValue);
  20 : Result := CreateTag(XML_RESOLUTION, AValue);
  21 : Result := CreateTag(XML_FRAMERATE, AValue);
  22 : Result := CreateTag(XML_VIDEOCODEC, AValue);
  23 : Result := CreateTag(XML_VIDEOBITRATE, AValue);
  24 : Result := CreateTag(XML_AUDIOCODEC, AValue);
  25 : Result := CreateTag(XML_AUDIOBITRATE, AValue);
  26 : Result := CreateTag(XML_SIZE, AValue);
  27 : Result := CreateTag(XML_PATH, AValue);
  28 : Result := CreateTag(XML_MEDIATYPE, AValue);
  29 : Result := CreateTag(XML_MEDIACOUNT, AValue);
  30 : Result := CreateTag(XML_FEATURES, AValue);
  31 : Result := CreateTag(XML_BARCODE, AValue);
  32 : Result := CreateTag(XML_WISH, AValue);
  33 : Result := CreateTag(XML_BOOKMARKED, AValue);
  34 : Result := CreateTag(XML_GENRE, AValue);
  35 : Result := CreateTag(XML_COUNTRY, AValue);
  36 : Result := CreateTag(XML_CATEGORY, AValue);
  37 : Result := CreateTag(XML_MEDIALABEL, AValue);
  38 : Result := CreateTag(XML_LANGUAGES, AValue);
  39 : Result := CreateTag(XML_SUBS, AValue);
  40 : Result := CreateTag(XML_STUDIO, AValue);
  41 : Result := CreateTag(XML_TAGS, AValue);
  42 : Result := CreateTag(XML_VIEWED, AValue);
  43 : Result := CreateTag(XML_VIEWDATE, AValue);
  44 : begin
        Result := CreateTag('directors', '', True, False);
        ExplodeString2(AValue, V);
        for I := High(V) downto Low(V) do
         Result := Result + CreateTag('person', CreateTag('name', V[I]), True, True, False);
        Result := Result + CreateTag('directors', '', False, True);
       end;
  45 : begin
        Result := CreateTag('writers', '', True, False);
        ExplodeString2(AValue, V);
        for I := High(V) downto Low(V) do
         Result := Result + CreateTag('person', CreateTag('name', V[I]), True, True, False);
        Result := Result + CreateTag('writers', '', False, True);
       end;
  46 : begin
        Result := CreateTag('producers', '', True, False);
        ExplodeString2(AValue, V);
        for I := High(V) downto Low(V) do
         Result := Result + CreateTag('person', CreateTag('name', V[I]), True, True, False);
        Result := Result + CreateTag('producers', '', False, True);
       end;
  47 : begin
       Result := CreateTag('composers', '', True, False);
       ExplodeString2(AValue, V);
       for I := High(V) downto Low(V) do
        Result := Result + CreateTag('person', CreateTag('name', V[I]), True, True, False);
       Result := Result + CreateTag('composers', '', False, True);
      end;
  48 : begin
       Result := CreateTag('actors', '', True, False);
       ExplodeString2(AValue, V);
       for I := High(V) downto Low(V) do
        Result := Result + CreateTag('person', CreateTag('name', V[I]), True, True, False);
       Result := Result + CreateTag('actors', '', False, True);
      end;
  else
   Result := '';
 end;
end;

function CreateCustomTag(FieldID : Integer; AValue : WideString) : WideString;
begin
 if AValue <> '' then
  Result := CreateTag(XML_CUSTOMVALUE, CreateTag(XML_CUSTOMID, IntToStr(FieldID), True, True, False) + CreateTag(XML_CUSTOM, AValue), True, True, False)
 else
  Result := '';
end;


function ExplodeString2(AText : WideString; var Items : TWideArray; const Delimiters : WideString) : Integer;
var
 I : Integer;
 S : WideString;
begin
 Result := 0;
 SetLength(Items, Result);

 for I := Length(AText) downto 1 do
  if Pos(AText[I], Delimiters) > 0 then begin
   if (I > 1) AND (Pos(AText[I-1], Delimiters) > 0) then begin
    Delete(AText, I, 1);
   end else begin
    S := Trim(Copy(AText, I + 1, Length(AText) - I));

    if S <> '' then begin
     Inc(Result);
     SetLength(Items, Result);
     Items[Result - 1] := S;
    end;

    Delete(AText, I, Length(AText) - I + 1);
   end;
  end;

 AText := Trim(AText);
 if AText <> '' then begin
  Inc(Result);
  SetLength(Items, Result);
  Items[Result - 1] := Trim(AText);
 end;
end;

function ExplodeCustomFields(AText : PWideChar; var Fields : TWideArray; var Indexes : TIntArray) : Integer; overload;
var
 I : Integer;
 S : WideString;
 IsField, Done : Boolean;
begin
 Result := 0;
 SetLength(Fields, Result);
 SetLength(Indexes, Result);

 if AText = nil then
  Exit;

 I := 0;
 Done := (AText[I] = #8) AND (AText[I + 1] = #0);
 IsField := True;

 while not Done do begin
  S := '';
  while AText[I] <> #0 do begin
   S := S + AText[I];
   Inc(I);
  end;

  if IsField then begin
   Inc(Result);
   SetLength(Fields, Result);
   SetLength(Indexes, Result);
  end;

  if IsField then
   Fields[Result - 1] := S
  else
   Indexes[Result - 1] := StrToIntDef(S, 0);

   IsField := not IsField;

  Inc(I);
  if (AText[I] = #8) AND (AText[I + 1] = #0) then
   Done := True;
 end;
end;

function FloatToStrSQL(const AValue : Extended) : String;
var
 tmp : Char;
begin
 tmp := DecimalSeparator;
 DecimalSeparator := '.';
 Result := FloatToStr(AValue);
 DecimalSeparator := tmp;
end;

function FloatStrToSQL(const AValue : String) : String;
var
 F : Extended;
 tmp : Char;
begin
 try
  F := StrToFloat(AValue);
 except
  tmp := DecimalSeparator;
  DecimalSeparator := '.';
  F := StrToFloatDef(AValue, 0);
  DecimalSeparator := tmp;
 end;

 Result := FloatToStrSQL(F);
end;

function FloatStrToFloat(const AValue : String) : Extended;
var
 tmp : Char;
begin
 try
  Result := StrToFloat(AValue);
 except
  tmp := DecimalSeparator;
  DecimalSeparator := '.';
  Result := StrToFloatDef(AValue, 0);
  DecimalSeparator := tmp;
 end;
end;

function DateToStrSQL(const AValue : TDateTime) : String;
begin
 Result := FormatDateTime('YYYY-MM-DD', AValue);
end;

function TimeToStrSQL(const AValue : TDateTime) : String;
begin
 Result := FormatDateTime('HH:NN:SS', AValue);
end;

function SecondsToTimeString(AValue : Integer) : String;
var
 N : Integer;
begin
 Result := '';

 N := AValue div 86400;
 if N > 0 then begin
  Result := Result + IntToStr(N) + 'D ';
  AValue := AValue - (N * 86400);
 end;

 N := AValue div 3600;
 if N < 10 then
  Result := Result + '0' + IntToStr(N) + ':'
 else
  Result := Result + IntToStr(N) + ':';
 AValue := AValue - (N * 3600);


 N := AValue div 60;
 if N < 10 then
  Result := Result + '0' + IntToStr(N) + ':'
 else
  Result := Result + IntToStr(N) + ':';
 AValue := AValue - (N * 60);

 if AValue < 10 then
  Result := Result + '0' + IntToStr(AValue)
 else
  Result := Result + IntToStr(AValue);
end;

function TimeStringToSeconds(AValue : String) : Integer;
var
 P : Integer;
begin
 Result := 0;

 AValue := Trim(AValue);

 P := Pos('D', LowerCase(AValue));
 if P > 0 then begin
  Result := Result + StrToIntDef(Copy(AValue, 1, P - 1), 0) * 86400;
  Delete(AValue, 1, P);
  AValue := Trim(AValue);
 end;

 P := Pos(':', AValue);
 if P < 1 then
  Exit;

 Result := Result + StrToIntDef(Copy(AValue, 1, P - 1), 0) * 3600;
 Delete(AValue, 1, P);

 P := Pos(':', AValue);
 if P < 1 then
  Exit;

 Result := Result + StrToIntDef(Copy(AValue, 1, P - 1), 0) * 60;
 Delete(AValue, 1, P);

 Result := Result + StrToIntDef(AValue, 0);
end;

function TimeToSeconds(AValue : TDateTime) : Integer;
var
 H, M, S, MS : Word;
begin
 DecodeTime(AValue, H, M, S, MS);

 Result := H * 3600 + M * 60 + S;
end;

function StringToIntIgnore(AString : String) : Integer;
var
 I : Integer;
begin
 AString := Trim(AString);

 for I := Length(AString) downto 1 do begin
  try
   StrToInt(AString[I]);
  except
   Delete(AString, I, 1);
  end;
 end;

 try
  Result := StrToInt(Trim(AString));
 except
  Result := 0;
 end;
end;

function SizeStringToMBIntIgnore(AString : String) : Integer;
const
  SizeNamesAr : array [0..2] of String = ('GB', 'MB', 'KB');
var
 I, P : Integer;
 E : Extended;
begin
 AString := AnsiUpperCase(Trim(AString));

 for I := Low(SizeNamesAr) to High(SizeNamesAr) do begin
  P := Pos(SizeNamesAr[I], AString);
  if P > 1 then begin
   Delete(AString, P, Length(SizeNamesAr[I]));

   try
    E := StrToFloat(Trim(AString));
   except
    E := 0;
   end;

   if E > 0 then begin
    case I of
     0 : Result := Ceil(E * 1024);
     1 : Result := Ceil(E);
     2 : Result := Ceil(E / 1024);
    else
     Result := Ceil(E);
    end;

    Exit;
   end;

   Break;
  end;
 end;

 Result := StringToIntIgnore(AString);
end;

function ParseURLs(AText : String; var URLs : TURLList) : Integer;
var
 P : Integer;
 S : String;
begin
 Result := 0;
 SetLength(URLs, Result);

 AText := Trim(AText);

 P := Pos(' ', AText);
 while P > 0 do begin
  S := Trim(Copy(AText, 1, P - 1));
  if S <> '' then begin
   Inc(Result);
   SetLength(URLs, Result);
   URLs[Result - 1] := S;
  end;
  Delete(AText, 1, P);
  P := Pos(' ', AText);
 end;

 AText := Trim(AText);
 if AText <> '' then begin
  Inc(Result);
  SetLength(URLs, Result);
  URLs[Result - 1] := AText;
 end;
end;

procedure ClearHuman(var Human : THumanRec);
begin
 Human.Name := '';
 Human.TransName := '';
 Human.AltNames := '';
 Human.Birthday := '';
 Human.Birthplace := '';
 Human.Genre := '';
 Human.URL := '';
 Human.Bio := '';
 Human.Films := '';
 Human.Awards := '';
 Human.DeathDate := '';
end;

procedure ClearMovie(var Movie : TMovieRec);
begin
 Movie.Title := '';
 Movie.OrigTitle := '';
 Movie.Aka := '';
 Movie.Year := 0;
 Movie.Genre := '';
 Movie.Director := '';
 Movie.Producer := '';
 Movie.Scenario := '';
 Movie.Studio := '';
 Movie.Music := '';
 Movie.Actors := '';
 Movie.Country := '';
 Movie.MPAA := '';
 Movie.Rating := 0;
 Movie.RatingName := '';
 Movie.Tags := '';
 Movie.Tagline := '';
 Movie.Description := '';
 Movie.Duration := 0;
 Movie.Awards := '';
 Movie.Links := '';
 Movie.URL := '';
 Movie.Features := '';
 Movie.Episodes := '';
end;

function URLEncodeWide(const S : WideString; CP : Cardinal) : String;
begin
 if CP > 0 then
  Result := WideStringToMultibyte(S, CP)
 else
  Result := S;

 Result := URLEncode(Result);
end;

function HTMLToWideString(const HTML : String; DefCP : Cardinal) : WideString;
var
 P, P2 : Integer;
 charset : String;
 CP : Cardinal;
begin
 CP := DefCP;

 P := Pos('http-equiv="Content-Type"', HTML);
 if P < 1 then
  P := Pos('http-equiv="content-type"', HTML);
 if P > 0 then begin
  P := PosEx('>', HTMl, P);
  if P > 0 then begin
   P2 := Pos('charset=', HTML);

   if (P2 > 0) AND (P2 < P) then begin
    P2 := P2 + Length('charset=');
    P := PosEx('"', HTML, P2);

    if P > 0 then
     charset := Copy(HTML, P2, P - P2);
   end;
  end;
 end;

 if CompareText('iso-8859-1', charset) = 0 then
  CP := 28591
 else
 if CompareText('iso-8859-2', charset) = 0 then
  CP := 28592
 else
 if CompareText('iso-8859-3', charset) = 0 then
  CP := 28593
 else
 if CompareText('iso-8859-4', charset) = 0 then
  CP := 28594
 else
 if CompareText('iso-8859-5', charset) = 0 then
  CP := 28595
 else
 if CompareText('iso-8859-6', charset) = 0 then
  CP := 28596
 else
 if CompareText('iso-8859-7', charset) = 0 then
  CP := 28597
 else
 if CompareText('iso-8859-8', charset) = 0 then
  CP := 28598
 else
 if CompareText('iso-8859-9', charset) = 0 then
  CP := 28599
 else
 if CompareText('windows-1250', charset) = 0 then
  CP := 1250
 else
 if CompareText('windows-1251', charset) = 0 then
  CP := 1251
 else
 if CompareText('windows-1252', charset) = 0 then
  CP := 1252
 else
 if CompareText('windows-1253', charset) = 0 then
  CP := 1253
 else
 if CompareText('windows-1254', charset) = 0 then
  CP := 1254
 else
 if CompareText('windows-1255', charset) = 0 then
  CP := 1255
 else
 if CompareText('windows-1256', charset) = 0 then
  CP := 1256
 else
 if CompareText('windows-1257', charset) = 0 then
  CP := 1257
 else
 if CompareText('windows-1258', charset) = 0 then
  CP := 1258
 else
 if CompareText('macintosh', charset) = 0 then
  CP := 1000
 else
 if CompareText('x-mac-japanese', charset) = 0 then
  CP := 1001
 else
 if CompareText('x-mac-chinesetrad', charset) = 0 then
  CP := 1002
 else
 if CompareText('x-mac-korean', charset) = 0 then
  CP := 1003
 else
 if CompareText('x-mac-arabic', charset) = 0 then
  CP := 1004
 else
 if CompareText('x-mac-hebrew', charset) = 0 then
  CP := 1005
 else
 if CompareText('x-mac-greek', charset) = 0 then
  CP := 1006
 else
 if CompareText('x-mac-cyrillic', charset) = 0 then
  CP := 1007
 else
 if CompareText('x-mac-chinesesimp', charset) = 0 then
  CP := 1008
 else 
 if CompareText('gb2312', charset) = 0 then
  CP := 936
 else
 if CompareText('shift_jis', charset) = 0 then
  CP := 932
 else
 if CompareText('cp875', charset) = 0 then
  CP := 875
 else
 if CompareText('ks_c_5601-1987', charset) = 0 then
  CP := 949
 else
 if CompareText('big5', charset) = 0 then
  CP := 950
 else
 if CompareText('windows-874', charset) = 0 then
  CP := 874
 else
 if CompareText('Johab', charset) = 0 then
  CP := 1361
 else
 if CompareText('x-mac-romanian', charset) = 0 then
  CP := 10010
 else
 if CompareText('x-mac-ukrainian', charset) = 0 then
  CP := 10017
 else
 if CompareText('x-mac-thai', charset) = 0 then
  CP := 10021
 else
 if CompareText('x-mac-ce', charset) = 0 then
  CP := 10029
 else
 if CompareText('x-mac-icelandic', charset) = 0 then
  CP := 10079
 else
 if CompareText('x-mac-turkish', charset) = 0 then
  CP := 10081
 else
 if CompareText('x-mac-croatian', charset) = 0 then
  CP := 10082
 else
 if CompareText('koi8-r', charset) = 0 then
  CP := 20866
 else
 if CompareText('EUC-JP', charset) = 0 then
  CP := 20932
 else
 if CompareText('x-cp20936', charset) = 0 then
  CP := 20936
 else
 if CompareText('x-cp20949', charset) = 0 then
  CP := 20949
 else
 if CompareText('koi8-u', charset) = 0 then
  CP := 21866
 else
 if CompareText('iso-8859-13', charset) = 0 then
  CP := 28603
 else
 if CompareText('iso-8859-15', charset) = 0 then
  CP := 28605
 else
 if CompareText('x-Europa', charset) = 0 then
  CP := 29001
 else
 if CompareText('iso-8859-8-i', charset) = 0 then
  CP := 38598
 else
 if CompareText('iso-2022-jp', charset) = 0 then
  CP := 50220
 else
 if CompareText('csISO2022JP', charset) = 0 then
  CP := 50221
 else
 if CompareText('iso-2022-jp', charset) = 0 then
  CP := 50222
 else
 if CompareText('iso-2022-kr', charset) = 0 then
  CP := 50225
 else
 if CompareText('euc-jp', charset) = 0 then
  CP := 51932
 else
 if CompareText('EUC-CN', charset) = 0 then
  CP := 51936
 else
 if CompareText('euc-kr', charset) = 0 then
  CP := 51949;


 if CP > 0 then
  Result := ConvertEncoding(HTML, CP)
 else
  Result := HTML;
end;

function ConvertEncoding(const HTML : String; CP : Cardinal) : WideString;
var
 L, uL : Integer;
begin
 L := Length(HTML) + 1;
 ul := L * 2;

 SetLength(Result, uL);
 MultiByteToWideChar(CP, 0, PCHAR(HTML), L, PWidechar(Result), uL);
end;

function WideStringToMultibyte(const AText : WideString; CP : Cardinal) : String;
var
 L1, L2 : Integer;
begin
 L1 := Length(AText);
 L2 := WideCharToMultiByte(CP, 0, PWideChar(AText), -1, nil, 0, nil, nil) - 1;
 SetLength(Result, L2);
 WideCharToMultiByte(CP, 0, PWideChar(AText), L1, PCHAR(Result), L2, nil, nil);
end;

function URLEncode(const S: String; const InQueryString: Boolean): String;
var
  Idx: Integer; // loops thru characters in string
begin
  Result := '';
  for Idx := 1 to Length(S) do
  begin
    case S[Idx] of
      'A'..'Z', 'a'..'z', '0'..'9', '-', '_', '.':
        Result := Result + S[Idx];
      ' ':
        if InQueryString then
          Result := Result + '+'
        else
          Result := Result + '%20';
      else
        Result := Result + '%' + SysUtils.IntToHex(Ord(S[Idx]), 2);
    end;
  end;
end;

function URLDecode(const S: String): String;
var
  Idx: Integer;   // loops thru chars in string
  Hex: string;    // string of hex characters
  Code: Integer;  // hex character code (-1 on error)
begin
  // Intialise result and string index
  Result := '';
  Idx := 1;
  // Loop thru string decoding each character
  while Idx <= Length(S) do
  begin
    case S[Idx] of
      '%':
      begin
        // % should be followed by two hex digits - exception otherwise
        if Idx <= Length(S) - 2 then
        begin
          // there are sufficient digits - try to decode hex digits
          Hex := S[Idx+1] + S[Idx+2];
          Code := SysUtils.StrToIntDef('$' + Hex, -1);
          Inc(Idx, 2);
        end
        else
          // insufficient digits - error
          Code := -1;
        // check for error and raise exception if found
        if Code = -1 then
          raise SysUtils.EConvertError.Create(
            'Invalid hex digit in URL'
          );
        // decoded OK - add character to result
        Result := Result + Chr(Code);
      end;
      '+':
        // + is decoded as a space
        Result := Result + ' '
      else
        // All other characters pass thru unchanged
        Result := Result + S[Idx];
    end;
    Inc(Idx);
  end;
end;

function TextToHTML(TheText: string): string;
var
  Idx: Integer; // loops thru the given text
begin
  Result := '';
  for Idx := 1 to Length(TheText) do
    case TheText[Idx] of
      '<':  // opens tags: replace with special char reference
        Result := Result + '&lt;';
      '>':  // closes tags: replace with special char reference
        Result := Result + '&gt;';
      '&':  // begins char references: replace with special char reference
        Result := Result + '&amp;';
      '"':  // quotes (can be a problem in quoted attributes)
        Result := Result + '&quot;';
      #0..#31, #127..#255:  // control and special chars: replace with encoding
        Result := Result + '&#' + SysUtils.IntToStr(Ord(TheText[Idx])) + ';';
      else  // compatible text: pass thru
        Result := Result + TheText[Idx];
    end;
end;

function HTMLToText(const HTML : WideString) : WideString;
var
 P, P1 : Integer;
 C : WideChar;
begin
 Result := HTML;
 P := Pos('&#', Result);
 while P > 0 do begin
  P1 := Pos(';', Copy(Result, P, Length(Result) - P + 1)) + (P - 1);
  if P1 > 0 then begin
   try
    C := WideChar(StrToInt(Copy(Result, P + 2, P1 - (P + 2))));
   except
    C := #0;
   end;

   if C <> #0 then begin
    Delete(Result, P, P1 - P + 1);
    Insert(C, Result, P);
   end;
   P := Pos('&#', Result);
  end else
   P := Pos('&#', Copy(Result, P + 2, Length(Result) - (P + 1))) + (P - 1);
 end;

 Result := Tnt_WideStringReplace(Result, '<br>', #13, [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '<br/>', #13, [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '<br />', #13, [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '<nobr>', '', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '</nobr>', '', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&nbsp;', ' ', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&hellip;', '...', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&mdash;', '—', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&amp;', '&', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&laquo;', '"', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&raquo;', '"', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&quot;', '"', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&euml;', 'ë', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&auml;', 'ä', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&ouml;', 'ö', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&uuml;', 'ü', [rfReplaceAll]);
 Result := Tnt_WideStringReplace(Result, '&ordm;', '°', [rfReplaceAll]);    
end;

function MinutesToTimeStr(Minutes : Integer; SQL : Boolean) : String;
var
 Time : TDateTime;
begin
 Time := EncodeTime(Minutes div 60, Minutes mod 60, 0, 0);

 if SQL then
  Result := TimeToStrSQL(Time)
 else
  Result := TimeToStr(Time);
end;

function TimeStrToSQL(AString : String) : String;
begin
 Result := FormatDateTime('hh:nn:ss', TimeStrSQLToTime(AString));
end;

function DateStrToSQL(AString : String) : String;
begin
 Result := FormatDateTime('YYYY-MM-DD', StrToDateDef(AString, 0));
end;

function TimeStrSQLToTime(AString : String) : TDateTime;
var
 Format : TFormatSettings;
begin
 GetLocaleFormatSettings(GetThreadLocale, Format);

 Format.TimeSeparator := ':';
 Format.ShortTimeFormat := 'hh:nn:ss';
 Format.LongTimeFormat := 'hh:nn:ss';

 try
  Result := StrToTime(AString, Format);
 except
  Result := StrToTimeDef(AString, 0);
 end;
end;

function MinutesStrToTimeStr(Minutes : String; SQL : Boolean) : String;
begin
 try
  Result := MinutesToTimeStr(StrToInt(Trim(Minutes)), SQL);
 except
  Result := Minutes;
 end;
end;

function MinutesToTime(Minutes : Integer) : TDateTime;
begin
 Result := EncodeTime(Minutes div 60, Minutes mod 60, 0, 0);
end;

function MinutesStrToTime(Minutes : String) : TDateTime;
begin
 try
  Result := MinutesToTime(StrToInt(Trim(Minutes)));
 except
  Result := 0;
 end;
end;

function TimeToMinutesStr(Time : TDateTime) : String;
var
 H, M, S : Word;
begin
 DecodeTime(Time, H, M, S, S);
 Result := IntToStr(H * 60 + M);
end;

end.
