﻿library csvimp;

uses
  Windows,
  Forms,
  Controls,
  SysUtils,
  Classes,
  ImportU,
  TntSysUtils,
  csvConfigFormU in 'csvConfigFormU.pas' {csvConfigForm: TTntForm},
  XMLGenU;

const
  PLUGIN_NAME    = 'CSV';
  PLUGIN_DESC_EN = 'Get Movies from CSV file';
  PLUGIN_DESC_DE = 'Filme aus einer CSV Datei importieren';
  PLUGIN_DESC_RU = 'Импортировать фильмы из CVS';
  PLUGIN_DESC_ES = 'Obtener información de CVS';
  PLUGIN_TYPE    = 3;
  PLUGIN_SUPVER  = 9008007;
  PLUGIN_VER     = '0.1.1.2';
  PLUGIN_LANG    = LANG_ENGLISH;
  PVD_SUPVER_MIN = 9008007;

  PLUGIN_CONFIG_FILENAME = 'Plugins\csvconf.cfg';

  FILE_FILTER = 'CSV Files|*.csv';

  GROUP_EN    = 'Options';
  GROUP_RU    = 'Настройки';
  GROUP_DE    = 'Einstellungen';
  DELIM_EN    = 'Delimiter:';
  DELIM_RU    = 'Разделитель:';
  DELIM_DE    = 'Trennzeichen:';
  QUAL_EN     = 'Text Qualifier:';
  QUAL_RU     = 'Ограничитель текста:';
  QUAL_DE     = 'Textbegrenzer:';
  HEADIGN_EN  = 'Ignore header';
  HEADIGN_RU  = 'Игнорировать первую строку';
  HEADIGN_DE  = 'Erste Zeile ignorieren';


  FIELDS_COUNT = 49;


type
  TCats = array of Integer;
  TSplitResult = array of WideString;

var
 Cats : TCats;
 Delimiter : Char;
 Qualifier : Char;
 IgnoreHead : Boolean;

{$R *.res}

function GetName(const LangID : Integer) : PWideChar; stdcall;
begin
 Result := PLUGIN_NAME;
end;

function GetDescription(const LangID : Integer) : PWideChar; stdcall;
begin
 case LangID of
  LANG_RUSSIAN : Result := PLUGIN_DESC_RU;
  LANG_GERMAN : Result := PLUGIN_DESC_DE;
  LANG_SPANISH : Result := PLUGIN_DESC_ES;
  else
   Result := PLUGIN_DESC_EN;
 end;
end;

function GetType : SHORT; stdcall;
begin
 Result := PLUGIN_TYPE;
end;

function GetPluginLanguage : Integer; stdcall;
begin
 Result := PLUGIN_LANG;
end;

(*
Use 0 for all
*)
function GetVersionSupported : DWORD; stdcall;
begin
 Result := PLUGIN_SUPVER;
end;

function CheckPVDVerion(const V : Integer) : BOOL; stdcall;
begin
 Result := PVD_SUPVER_MIN <= V;
end;


function GetPluginVersion : PCHAR; stdcall;
begin
 Result := PLUGIN_VER;
end;

function CanConfig : BOOL; stdcall;
begin
 Result := True;
end;

procedure GetConfigData;
var
 F : TextFile;
 S : String;
 N : Integer;
begin
 for N := Low(Cats) to High(Cats) do
  Cats[N] := -1;

 if FileExists(ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME) then begin
   AssignFile(F, ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME);
  FileMode := 1;
  Reset(F);
  ReadLn(F, S);
  if S <> '' then
   Delimiter := S[1]
  else
   Delimiter := #0;
  ReadLn(F, S);
  if S <> '' then
   Qualifier := S[1]
  else
   Qualifier := #0;
  ReadLn(F, S);
  try
   IgnoreHead := StrToBool(S);
  except
   IgnoreHead := True;
  end; 

  N := Low(Cats);
  while (not EOF(F)) AND (N < Length(Cats)) do begin
   ReadLn(F, S);
   try
    Cats[N] := StrToInt(S);
   except
    Cats[N] := -1;
   end;

   Inc(N);
  end;

  CloseFile(F);
 end;
end;

function Configure(const LangID : Integer; AppHandle : HWND; CustomFields : PWideChar) : BOOL; stdcall;
var
 F : TextFile;
 csvConfFrm : TcsvConfigForm;
 I, N : Integer;
 V : TWideArray;
 IDs : TIntArray;
begin
 Result := False;

 SetLength(Cats, FIELDS_COUNT + ExplodeCustomFields(CustomFields, V, IDs));
 GetConfigData;

 Application.Handle := AppHandle;
 csvConfFrm := TcsvConfigForm.Create(Application);
 try
  csvConfFrm.CatList.Items.Delimiter := ',';
  csvConfFrm.CatList.Items.StrictDelimiter := True;

  case LangID of
   LANG_RUSSIAN : begin
                   csvConfFrm.OptsGroup.Caption := GROUP_RU;
                   csvConfFrm.DelimLbl.Caption := DELIM_RU;
                   csvConfFrm.QualLbl.Caption := QUAL_RU;
                   csvConfFrm.IgnoreCheck.Caption := HEADIGN_RU;
                  end;
   LANG_GERMAN : begin
                  csvConfFrm.OptsGroup.Caption := GROUP_DE;
                  csvConfFrm.DelimLbl.Caption := DELIM_DE;
                  csvConfFrm.QualLbl.Caption := QUAL_DE;
                  csvConfFrm.IgnoreCheck.Caption := HEADIGN_DE;
                 end
   else begin
    csvConfFrm.OptsGroup.Caption := GROUP_EN;
    csvConfFrm.DelimLbl.Caption := DELIM_EN;
    csvConfFrm.QualLbl.Caption := QUAL_EN;
    csvConfFrm.IgnoreCheck.Caption := HEADIGN_EN;
   end;
  end;
  for I := Low(V) to High(V) do
   csvConfFrm.CatList.Items.Add(V[I]);

  csvConfFrm.SelList.Clear;
  for I := Low(Cats) to High(Cats) do
   if (Cats[I] > -1) AND (Cats[I] < csvConfFrm.CatList.Items.Count) then
    csvConfFrm.SelList.Items.Add(csvConfFrm.CatList.Items[Cats[I]]);

  if Delimiter <> #0 then
   csvConfFrm.DelimEdit.Text := Delimiter
  else
   csvConfFrm.DelimEdit.Text := '';
  if Qualifier <> #0 then
   csvConfFrm.QualEdit.Text := Qualifier
  else
   csvConfFrm.QualEdit.Text := ''; 
  csvConfFrm.IgnoreCheck.Checked := IgnoreHead;

  if csvConfFrm.ShowModal = mrOK then begin
   Delimiter := csvConfFrm.Delimiter;
   Qualifier := csvConfFrm.Qualifier;
   IgnoreHead := csvConfFrm.IgnoreCheck.Checked;

   AssignFile(F, ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME);
   FileMode := 2;
   Rewrite(F);
   if Delimiter <> #0 then
    WriteLn(F, Delimiter)
   else
    WriteLn(F, '');
   if Qualifier <> #0 then
    WriteLn(F, Qualifier)
   else
    WriteLn(F, '');
   WriteLn(F, BoolToStr(IgnoreHead));

   for I := 0 to csvConfFrm.SelList.Items.Count - 1 do begin
    N := csvConfFrm.CatList.Items.IndexOf(csvConfFrm.SelList.Items[I]);
    WriteLn(F, IntToStr(N));
   end;
   CloseFile(F);

   Result := True;
  end;
 finally
  csvConfFrm.Free;
 end;
end;

function GetFilter(AppHandle : HWND; const LangID : Integer; CustomFields : PWideChar) : PWideChar; stdcall;
begin
 Result := FILE_FILTER;

 if not FileExists(ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME) then
  if not Configure(LangID, AppHandle, CustomFields) then
   Result := nil;
end;

procedure Parse(AppHandle : HWND; const LangID : Integer; FileName : PWideChar; MovieCallback : TPluginImpGenMovieCallback; HumanCallback : TPluginImpGenPersonCallback; CustomFields : PWideChar); stdcall;
var
 N, P1, P2, ColIndex : Integer;
 Tmp, Line, Data : String;
 SkipIndex : Boolean;

 Movie : WideString;
 V : TWideArray;
 IDs : TIntArray;
 vN : Integer;
 F : TextFile;
begin
 if not WideFileExists(FileName) then
  Exit;

 SetLength(Cats, FIELDS_COUNT + ExplodeCustomFields(CustomFields, V, IDs));
 GetConfigData;

 if (Delimiter = #0) AND (Qualifier = #0) then
  Exit;

 AssignFile(F, FileName);
 FileMode := 1;
 Reset(F);
 try
  if IgnoreHead then
   ReadLn(F, Data);

  Tmp := '';

  while not EOF(F) do begin
   ReadLn(F, Line);
   if Length(Line) < 1 then
    Continue;

   Movie := CreateTag('xml', '', True, False);
   Movie := Movie + CreateTag('movie', '', True, False);

   ColIndex := 0;
   N := 1;
   Data := '';

   while (N < Length(Line)) AND (N > 0) do begin
    SkipIndex := False;

    P1 := PosFrom(Delimiter, Line, N);
    P2 := PosFrom(Qualifier, Line, N);
    if ((P2 > 0) AND (P2 < P1)) OR
       ((P1 = 0) AND (P2 > 0) AND (PosFrom(Qualifier, Line, P2 + 1) > 0)) then begin
     P1 := PosFrom(Qualifier, Line, P2 + 1);
     Data := Copy(Line, P2 + 1, P1 - P2 - 1);
     N := PosFrom(Delimiter, Line, P1 + 1);
     if N > 0 then
      Inc(N);
    end else
    if (P1 > 0) then begin
     Data := Copy(Line, N, P1 - N);
     N := P1 + 1;
    end else begin
     if (N < Length(Line)) AND (P2 > 0) AND (not EOF(F)) then begin
      Data := '';
      Tmp := Copy(Line, N, Length(Line) - N + 1);
      ReadLn(F, Line);
      Line := Tmp + #13#10 + Line;
      Tmp := '';
      SkipIndex := True;
      N := 1;
     end else begin
      Tmp := '';
      N := 0;
     end;
    end;

    if Data <> '' then begin
     if (Cats[ColIndex] > -1) AND (Cats[ColIndex] < FIELDS_COUNT) then
      Movie := Movie + CreateFieldTag(Cats[ColIndex], Data)
     else
     if (Cats[ColIndex] >= FIELDS_COUNT) then begin
      vN := Cats[ColIndex] - FIELDS_COUNT;
      if (vN >= Low(IDs)) AND (vN <= High(IDs)) then
       Movie := Movie + CreateCustomTag(IDs[vN], Data);
     end;
    end;

    if not SkipIndex then
     Inc(ColIndex);
   end;

   Movie := Movie + CreateTag('movie', '', False, True) + CreateTag('xml', '', False, True);

   if not MovieCallback(PWideChar(Movie)) then
    Exit;

   Movie := '';
  end;
 finally
  CloseFile(F);
 end;
end;

function BeginWork : Integer; stdcall;
begin
 Result := 0;
end;

procedure EndWork; stdcall;
begin

end;


exports
  BeginWork,
  EndWork,
  GetName,
  GetDescription,
  GetType,
  GetVersionSupported,
  GetPluginVersion,
  CanConfig,
  Configure,
  Parse,
  GetFilter,
  GetPluginLanguage,
  CheckPVDVerion;

begin
end.
