﻿library kinopoisk;

uses
  Windows,
  SysUtils,
  Controls,
  TntSysUtils,
  Forms,
  LoginConfigFormU in 'LoginConfigFormU.pas' {ConfigForm},
  ImportU,
  XMLGenU;

const
  wmFinished = 0;
  wmSearch = 1;
  wmMain = 2;
  wmCredits = 3;
  wmAwards = 4;
  wmLinks = 5;
  wmKeywords = 6;

type
  TWorkMode = wmFinished..wmKeywords;

const
  PLUGIN_NAME   = 'KinoPoisk [RU]';
  PLUGIN_DESC_EN   = '[RU] Get Movie Information from KinoPoisk.ru';
  PLUGIN_DESC_DE   = '[RU] Filmdaten aus KinoPoisk.ru importieren';
  PLUGIN_DESC_RU   = '[RU] Импортировать информацио о фильме из KinoPoisk.ru';
  PLUGIN_DESC_ES   = '[RU] Obtener información de película de KinoPoisk.ru';
  PLUGIN_TYPE   = PLUGIN_WEBIMPORT;
  PLUGIN_SUPVER = 9008000;
  PLUGIN_VER    = '0.4.1.3';
  PLUGIN_LANG   = LANG_RUSSIAN;
  PLUGIN_CONFIG_FILENAME = 'Plugins\kppconf.cfg';
  PVD_SUPVER_MIN = 9008000;

  KINOPOISK_BASE = 'http://www.kinopoisk.ru';
  KINOPOISK_SEARCH_STR = 'http://www.kinopoisk.ru/index.php?kp_query=%s';
  KINOPOISK_LOGIN_URL  = 'http://www.kinopoisk.ru/level/30/';

  CODE_PAGE = 1251;

  RATING_STR = 'KinoPoisk.ru';

  STR_LOGIN_EN    = 'Login:';
  STR_PASSWORD_EN = 'Password:';
  STR_LOGIN_RU    = 'Логин:';
  STR_PASSWORD_RU = 'Пароль:';
  STR_LOGIN_DE    = 'Benutzername:';
  STR_PASSWORD_DE = 'Passwort:';

var
 Selection : array of TSelectionRec;
 Movie : TMovieRec;
 Count : Integer = 0;

 ExtraLinks : array [wmCredits..wmKeywords] of String;
 URLs : TURLList;
 Mode : TWorkMode;

 //Options
 doTags   : Boolean = True; //TODO: False, write/read option and in UI
 doFullPersons : Boolean = True; //TODO: write/read option and in UI
 doConnections : Boolean = True; //TODO: False, write/read option and in UI
 doAwards      : Boolean = True; //TODO: False, write/read option and in UI
 doLinkFollowedOnly : Boolean = False; //TODO: True, write/read option and in UI

 XML : WideString;


{$R *.res}

function GetName(const LangID : Integer) : PWideChar; stdcall;
begin
 Result := PLUGIN_NAME;
end;

function GetDescription(const LangID : Integer) : PWideChar; stdcall;
begin
 case LangID of
  LANG_RUSSIAN : Result := PLUGIN_DESC_RU;
  LANG_GERMAN : Result := PLUGIN_DESC_DE;
  LANG_SPANISH : Result := PLUGIN_DESC_ES;
  else
   Result := PLUGIN_DESC_EN;
 end;
end;

function GetType : SHORT; stdcall;
begin
 Result := PLUGIN_TYPE;
end;

function GetPluginLanguage : Integer; stdcall;
begin
 Result := PLUGIN_LANG;
end;

function GetVersionSupported : DWORD; stdcall;
begin
 Result := PLUGIN_SUPVER;
end;

function CheckPVDVerion(const V : Integer) : BOOL; stdcall;
begin
 Result := PVD_SUPVER_MIN <= V;
end;

function GetPluginVersion : PCHAR; stdcall;
begin
 Result := PLUGIN_VER;
end;

function GetURL(const isEpisode : Boolean; const CurrentURL : PCHAR; out NewURL : PCHAR) : Integer; stdcall;
var
 I : Integer;
 Found : Boolean;
begin
 Result := WEB_URL_SEARCH;

 if Mode > wmMain then begin
  NewURL := PCHAR(ExtraLinks[Mode]);
  Result := WEB_URL_DIRECT;
 end else begin
  Found := False;
  for I := 0 to ParseURLs(CurrentURL, URLs) - 1 do
   if Pos('kinopoisk.ru', LowerCase(URLs[I])) > 0 then begin
    Found := True;
    NewURL := PCHAR(URLs[I]);
    Result := WEB_URL_DIRECT;
   end;

  if not Found then begin
   Result := WEB_URL_SEARCH;

   NewURL := KINOPOISK_SEARCH_STR;
  end; 
 end;
end;

function CanConfig : BOOL; stdcall;
begin
 Result := True;
end;

function GetReferrer : PCHAR; stdcall;
begin
 Result := KINOPOISK_BASE;
end;

function GetCodePage : Cardinal; stdcall;
begin
 Result := CODE_PAGE;
end;

procedure GetConfigData(var Login, Password : String);
var
 F : TextFile;
begin
 if FileExists(ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME) then begin
   AssignFile(F, ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME);
  FileMode := 1;
  Reset(F);
  ReadLn(F, Login);
  ReadLn(F, Password);
  CloseFile(F);
 end else begin
  Login := '';
  Password := '';
 end;
end;

function Configure(const LangID : Integer; AppHandle : HWND; CustomFields : PWideChar) : BOOL; stdcall;
var
 F : TextFile;
 ConfFrm : TConfigForm;
 L, P : String;
begin
 Result := False;

 GetConfigData(L, P);

 Application.Handle := AppHandle;
 ConfFrm := TConfigForm.Create(Application);
 try
  case LangID of
   LANG_RUSSIAN : begin
         ConfFrm.LoginLbl.Caption := STR_LOGIN_RU;
         ConfFrm.PassLbl.Caption := STR_PASSWORD_RU;
        end;
   LANG_GERMAN : begin
         ConfFrm.LoginLbl.Caption := STR_LOGIN_DE;
         ConfFrm.PassLbl.Caption := STR_PASSWORD_DE;
        end;
   else begin
    ConfFrm.LoginLbl.Caption := STR_LOGIN_EN;
    ConfFrm.PassLbl.Caption := STR_PASSWORD_EN;
   end;
  end;

  ConfFrm.LoginLbl.Left := ConfFrm.LoginEdit.Left - 6 - ConfFrm.LoginLbl.Width;
  ConfFrm.PassLbl.Left := ConfFrm.PassEdit.Left - 6 - ConfFrm.PassLbl.Width;

  ConfFrm.Caption := 'KinoPoisk.ru Account';

  ConfFrm.LoginEdit.Text := L;
  ConfFrm.PassEdit.Text := P;
  if ConfFrm.ShowModal = mrOK then begin
   AssignFile(F, ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME);
   FileMode := 2;
   Rewrite(F);
   WriteLn(F, ConfFrm.LoginEdit.Text);
   WriteLn(F, ConfFrm.PassEdit.Text);
   CloseFile(F);

   Result := True;
  end;
 finally
  ConfFrm.Free;
 end;
end;

procedure GetLoginInfo(out ScriptURL : PCHAR; out Params : PCHAR); stdcall;
var
 L, P : String;
begin
 ScriptURL := KINOPOISK_LOGIN_URL;

 GetConfigData(L, P);

 if (L <> '') AND (P <> '') then
  Params := PCHAR(Format('shop_user[login]=%s&shop_user[pass]=%s', [L, P]))
 else
  Params := PCHAR('');
end;

procedure ParseSearchResults(var HTML : WideString);
 procedure ParseTitles(var S : WideString);
 var
  P, P1, P2, E : Integer;
  Y, O, T : WideString;
 begin
  E := Pos('<!-- /результаты поиска -->', S);
  if E < 1 then
   Exit;
  P := Pos('href="/level/1/film/', S);
  while (P > 0) AND (P < E) do begin
   Delete(S, 1, P + Length('href="/level/1/film/') - 1);
   P1 := Pos('/', S);
   if P1 > 0 then begin
    Inc(Count);
    SetLength(Selection, Length(Selection) + 1);
    Selection[Length(Selection)-1].URL := 'http://www.kinopoisk.ru/level/1/film/'  + Copy(S, 1, P1);
    Selection[Length(Selection)-1].Browse := Selection[Length(Selection)-1].URL;
    P1 := Pos('">', S);
    P2 := Pos('</a>', S);
    if (P1 > 0) AND (P2 > 0) then begin
     T := Copy(S, P1 + 2, P2 - P1 - 2);
     T := Tnt_WideStringReplace(T, '(сериал)', '', [rfReplaceAll]);
     T := Tnt_WideStringReplace(T, '(ТВ)', '', [rfReplaceAll]);
     T := Tnt_WideStringReplace(T, '(видео)', '', [rfReplaceAll]);
     Selection[Length(Selection)-1].Title2 := T;
     Delete(S, 1, P2 + 3);

     O := '';
     Y := '';
     //Year
     P1 := Pos('<a href="/level/10/act/year/year/', S);
     if (P1 > 0) AND (P1 < 10) then begin
      Delete(S, 1, P1 + Length('<a href="/level/10/act/year/year/') - 1);
      P1 := Pos('>', S);
      P2 := Pos('</a>', S);
      if (P1 > 0) AND (P1 < P2) then begin
       Y := Copy(S, P1 + 1, P2 - P1 - 1);
       Delete(S, 1, P2 + Length('</a>') - 1);
      end;
     end;
     Selection[Length(Selection)-1].Year := StrToIntDef(Y, -1);

     //Original title
     P1 := Pos('<tr><td></td><td><font color="#999999">... ', S);
     if (P1 > 0) AND (P1 < 15) then begin
      Delete(S, 1, P1 + Length('<tr><td></td><td><font color="#999999">... ') - 1);
      P1 := Pos('</font>', S);
      if P1 > 0 then begin
       O := Copy(S, 1, P1 - 1);
       Delete(S, 1, P1 + Length('</font>') - 1);
      end;
     end;
     O := Tnt_WideStringReplace(O, '(сериал)', '', [rfReplaceAll]);
     O := Tnt_WideStringReplace(O, '(ТВ)', '', [rfReplaceAll]);
     O := Tnt_WideStringReplace(O, '(видео)', '', [rfReplaceAll]);
     Selection[Length(Selection)-1].Title := O;

     E := Pos('<!-- /результаты поиска -->', S);
    end;
   end;
   P := Pos('href="/level/1/film/', S);
  end;
 end;
var
 P, B, L : Integer;
begin
 Count := 0;
 SetLength(Selection, 0);

 B := High(Integer);
 L := 0;

 P := Pos('<b>Скорее всего Вы ищете:</b>', HTML);
 if P < B then begin
  B := P;
  L := Length('<b>Скорее всего Вы ищете:</b>');
 end;

 P := Pos('<b>Похожие результаты:</b>', HTML);
 if P < B then begin
  B := P;
  L := Length('<b>Похожие результаты:</b>');
 end;

 if B < High(Integer) then begin
  Delete(HTML, 1, B + L - 1);
  ParseTitles(HTML);
 end;
end;

procedure ParseMovie(var HTML : WideString);
var
 P1, P2, T, E : Integer;
 S, S2 : WideString;
begin
 ClearMovie(Movie);
 Count := 1;

 //Title
 P1 := Pos('<title>', HTML);
 P2 := Pos('</title>', HTML);
 if (P1 > 0) AND (P2 > 0) then begin
  Movie.Title := Copy(HTML, P1 + 7, P2 - (P1 + 7));
  Delete(HTML, 1, P2 + Length('</title>') - 1);
 end;

 //Original Title
 P1 := Pos('class="moviename-big">', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('class="moviename-big">') - 1);
  P1 := Pos('<span style=', HTML);
  if P1 > 0 then begin
   P1 := Pos('">', HTML);
   P2 := Pos('</span>', HTML);
   if (P1 > 0) AND (P2 > P1) then begin
    Movie.OrigTitle := Copy(HTML, P1 + 2, P2 - (P1 + 2));
    Delete(HTML, 1, P2 + Length('</span>') - 1);
   end;
  end;
 end;


 //Year
 P1 := Pos('год</td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('год</td>') - 1);
  P1 := Pos('<td class="desc-data"><a', HTML);
  if (P1 > 0) AND (P1 < 5) then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data"><a') - 1);
   P1 := Pos('">', HTML);
   P2 := Pos('</a></td>', HTML);
   if (P1 > 0) AND (P2 > P1) then begin
    try
     Movie.Year := StrToInt(Copy(HTML, P1 + 2, 4));
    except
     Movie.Year := 0;
    end;

    Delete(HTML, 1, P2 + Length('</a></td>') - 1);
   end;
  end;
 end;

 //Country
 P1 := Pos('страна</td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('страна</td>') - 1);
  P1 := Pos('<td class="desc-data">', HTML);
  if (P1 > 0) AND (P1 < 5) then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data">') - 1);
   T := Pos('<a href="/level/10/', HTML);
   E := Pos('</td>', HTML);
   while (T > 0) AND (T < E) do begin
    Delete(HTML, 1, T + Length('<a href="/level/10/') - 1);
    P1 := Pos('">', HTML);
    P2 := Pos('</a>', HTML);

    if (P1 > 0) AND (P2 > P1) then begin
     if Movie.Country <> '' then
      Movie.Country := Movie.Country + ', ';
     Movie.Country := Movie.Country + Copy(HTML, P1 + 2, P2 - (P1 + 2));

     Delete(HTML, 1, P2 + Length('</a>') - 1);
    end;

    T := Pos('<a href="/level/10/', HTML);
    E := Pos('</td>', HTML);
   end;

   Delete(HTML, 1, E + Length('</td>') - 1);
  end;
 end;

 //Tagline
 P1 := Pos('слоган</td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('слоган</td>') - 1);
  P1 := Pos('<td class="desc-data">', HTML);
  if (P1 > 0) AND (P1 < 5) then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data">') - 1);
   P1 := Pos('</td>', HTML);
   if (P1 > 0)  then begin
    Movie.Tagline := Trim(Copy(HTML, 1, P1 - 1));
    if Movie.Tagline = '-' then
     Movie.Tagline := '';

     Delete(HTML, 1, P1 + Length('</td>') - 1);
   end;
  end;
 end;

 //Director
 P1 := Pos('режиссер</td>', HTML);
 if P1 > 0 then begin
  Movie.Director := CreateTag('directors', '', True, False);
  try
   Delete(HTML, 1, P1 + Length('режиссер</td>') - 1);
   P1 := Pos('<td class="desc-data"><a', HTML);
   if (P1 > 0) AND (P1 < 5) then begin
    Delete(HTML, 1, P1 + Length('<td class="desc-data"><a') - 1);
    T := Pos(' href="/level/4/people/', HTML);
    E := Pos('</td>', HTML);
    while (T > 0) AND (T < E) do begin
     S := '';
     S2 := '';

     Delete(HTML, 1, T + Length(' href="/level/4/people/') - 1);
     P1 := Pos('/', HTML);
     if P1 > 0 then
      S := KINOPOISK_BASE + '/level/4/people/' + Copy(HTML, 1, P1);

     P1 := Pos('">', HTML);
     P2 := Pos('</a>', HTML);

     if (P1 > 0) AND (P2 > P1) then begin
      S2 := HTMLToText(Copy(HTML, P1 + 2, P2 - (P1 + 2)));
      Movie.Director := Movie.Director + CreateTag('person', CreateTag('transname', S2) + CreateTag('url', S), True, True, False);

      Delete(HTML, 1, P2 + Length('</a>') - 1);
     end;

     T := Pos(' href="/level/4/people/', HTML);
     E := Pos('</td>', HTML);
    end;

    Delete(HTML, 1, E + Length('</td>') - 1);
   end;
  finally
   Movie.Director := Movie.Director + CreateTag('directors', '', False, True);
  end;
 end;

 //Scenario
 P1 := Pos('сценарий</td>', HTML);
 if P1 > 0 then begin
  Movie.Scenario := CreateTag('writers', '', True, False);
  try
   Delete(HTML, 1, P1 + Length('сценарий</td>') - 1);
   P1 := Pos('<td class="desc-data"><a', HTML);
   if (P1 > 0) AND (P1 < 5) then begin
    Delete(HTML, 1, P1 + Length('<td class="desc-data"><a') - 1);
    T := Pos(' href="/level/4/people/', HTML);
    E := Pos('</td>', HTML);
    while (T > 0) AND (T < E) do begin
     S := '';
     S2 := '';

     Delete(HTML, 1, T + Length(' href="/level/4/people/') - 1);
     P1 := Pos('/', HTML);
     if P1 > 0 then
      S := KINOPOISK_BASE + '/level/4/people/' + Copy(HTML, 1, P1);

     P1 := Pos('">', HTML);
     P2 := Pos('</a>', HTML);

     if (P1 > 0) AND (P2 > P1) then begin
      S2 := HTMLToText(Copy(HTML, P1 + 2, P2 - (P1 + 2)));
      Movie.Scenario := Movie.Scenario + CreateTag('person', CreateTag('transname', S2) + CreateTag('url', S), True, True, False);

      Delete(HTML, 1, P2 + Length('</a>') - 1);
     end;

     T := Pos(' href="/level/4/people/', HTML);
     E := Pos('</td>', HTML);
    end;

    Delete(HTML, 1, E + Length('</td>') - 1);
   end;
  finally
   Movie.Scenario := Movie.Scenario + CreateTag('writers', '', False, True);
  end;
 end;

 //Producer
 P1 := Pos('продюсер</td>', HTML);
 if P1 > 0 then begin
  Movie.Producer := CreateTag('producers', '', True, False);
  try
   Delete(HTML, 1, P1 + Length('продюсер</td>') - 1);
   P1 := Pos('<td class="desc-data"><a', HTML);
   if (P1 > 0) AND (P1 < 5) then begin
    Delete(HTML, 1, P1 + Length('<td class="desc-data"><a') - 1);
    T := Pos(' href="/level/4/people/', HTML);
    E := Pos('</td>', HTML);
    while (T > 0) AND (T < E) do begin
     S := '';
     S2 := '';

     Delete(HTML, 1, T + Length(' href="/level/4/people/') - 1);
     P1 := Pos('/', HTML);
     if P1 > 0 then
      S := KINOPOISK_BASE + '/level/4/people/' + Copy(HTML, 1, P1);

     P1 := Pos('">', HTML);
     P2 := Pos('</a>', HTML);

     if (P1 > 0) AND (P2 > P1) then begin
      S2 := HTMLToText(Copy(HTML, P1 + 2, P2 - (P1 + 2)));
      Movie.Producer := Movie.Producer + CreateTag('person', CreateTag('transname', S2) + CreateTag('url', S), True, True, False);

      Delete(HTML, 1, P2 + Length('</a>') - 1);
     end;

     T := Pos(' href="/level/4/people/', HTML);
     E := Pos('</td>', HTML);
    end;

    Delete(HTML, 1, E + Length('</td>') - 1);
   end;
  finally
   Movie.Producer := Movie.Producer + CreateTag('producers', '', False, True);
  end;
 end;

 //Music
 P1 := Pos('композитор</td>', HTML);
 if P1 > 0 then begin
  Movie.Music := CreateTag('composers', '', True, False);
  try
   Delete(HTML, 1, P1 + Length('композитор</td>') - 1);
   P1 := Pos('<td class="desc-data"><a', HTML);
   if (P1 > 0) AND (P1 < 5) then begin
    Delete(HTML, 1, P1 + Length('<td class="desc-data"><a') - 1);
    T := Pos(' href="/level/4/people/', HTML);
    E := Pos('</td>', HTML);
    while (T > 0) AND (T < E) do begin
     S := '';
     S2 := '';

     Delete(HTML, 1, T + Length(' href="/level/4/people/') - 1);
     P1 := Pos('/', HTML);
     if P1 > 0 then
      S := KINOPOISK_BASE + '/level/4/people/' + Copy(HTML, 1, P1);

     P1 := Pos('">', HTML);
     P2 := Pos('</a>', HTML);

     if (P1 > 0) AND (P2 > P1) then begin
      S2 := HTMLToText(Copy(HTML, P1 + 2, P2 - (P1 + 2)));
      Movie.Music := Movie.Music + CreateTag('person', CreateTag('transname', S2) + CreateTag('url', S), True, True, False);

      Delete(HTML, 1, P2 + Length('</a>') - 1);
     end;

     T := Pos(' href="/level/4/people/', HTML);
     E := Pos('</td>', HTML);
    end;

    Delete(HTML, 1, E + Length('</td>') - 1);
   end;
  finally
   Movie.Music := Movie.Music + CreateTag('composers', '', False, True);
  end;
 end;

 //Genre(s)
 P1 := Pos('жанр</td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('жанр</td>') - 1);
  P1 := Pos('<td class="desc-data"><a', HTML);
  if (P1 > 0) AND (P1 < 5) then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data"><a') - 1);
   T := Pos(' href="/level/10/', HTML);
   E := Pos('</td>', HTML);
   while (T > 0) AND (T < E) do begin
    Delete(HTML, 1, T + Length(' href="/level/10/') - 1);
    P1 := Pos('">', HTML);
    P2 := Pos('</a>', HTML);

    if (P1 > 0) AND (P2 > P1) then begin
     if Movie.Genre <> '' then
      Movie.Genre := Movie.Genre + ', ';
     Movie.Genre := Movie.Genre + Copy(HTML, P1 + 2, P2 - (P1 + 2));

     Delete(HTML, 1, P2 + Length('</a>') - 1);
    end;

    T := Pos(' href="/level/10/', HTML);
    E := Pos('</td>', HTML);
   end;

   Delete(HTML, 1, E + Length('</td>') - 1);
  end;
 end;

 //MPAA     
 P1 := Pos('рейтинг MPAA</td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('рейтинг MPAA</td>') - 1);
  P1 := Pos('<td class="desc-data">', HTML);
  if (P1 > 0) AND (P1 < 5) then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data">') - 1);

   P1 := Pos(' href="/level/38/film/', HTML);
   Delete(HTML, 1, P1 + Length(' href="/level/38/film/') - 1);
   P1 := Pos('''>', HTML);
   P2 := Pos('</a>', HTML);

   if (P1 > 0) AND (P2 > P1) then begin
    Movie.MPAA := Copy(HTML, P1 + 2, P2 - (P1 + 2));

    Delete(HTML, 1, P2 + Length('</a>') - 1);
   end;
  end;
 end;

 //Duration
 P1 := Pos('время</td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('время</td>') - 1);
  P1 := Pos('<td class="desc-data">', HTML);
  if (P1 > 0) AND (P1 < 5) then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data">') - 1);
   P2 := Pos('</td>', HTML);
   if (P1 > 0) AND (P2 > P1) then begin
    try
     Movie.Duration := StrToInt(Trim(StringReplace(Copy(HTML, 1, P2 - 1), 'мин', '', [rfReplaceall])));
    except
     Movie.Duration := 0;
    end;

    Delete(HTML, 1, P2 + Length('</a></td>') - 1);
   end;
  end;
 end;

 //Actors
 Movie.Actors := CreateTag('actors', '', True, False);
 try
  P1 := Pos('В главных ролях:', HTML);
  Delete(HTML, 1, P1 + Length('В главных ролях:') - 1);
  T := Pos('<a href="/level/4/people/', HTML);
  while T > 0 do begin
   S := '';
   S2 := '';

   Delete(HTML, 1, T + Length('<a href="/level/4/people/') - 1);
   P1 := Pos('/', HTML);
   if P1 > 0 then
    S := KINOPOISK_BASE + '/level/4/people/' + Copy(HTML, 1, P1);

   P1 := Pos('">', HTML);
   P2 := Pos('</a>', HTML);
   if (P1 > 0) AND (P2 > P1) then begin
    S2 := HTMLToText(Copy(HTML, P1 + 2, P2 - (P1 + 2)));
    Movie.Actors := Movie.Actors + CreateTag('person', CreateTag('transname', S2) + CreateTag('url', S), True, True, False);

    Delete(HTML, 1, P2 + Length('</a>') - 1);
   end;
   T := Pos('<a href="/level/4/people/', HTML);
  end;
 finally
  Movie.Actors := Movie.Actors + CreateTag('actors', '', False, True);
 end;

 //Description
 P1 := Pos('class="news">', HTML);
 Delete(HTML, 1, P1 + Length('class="news">') - 1);
 P1 := Pos('</td>', HTML);
 P2 := Pos('<a href=', HTML);
 T := Pos('<table ', HTML);

 if (P2 < T) then
  T := P2;
 if (P1 < T) then
  T := P1;

 P1 := Pos('<ul class="trivia">', HTML);
 if (P1 = 0) OR (T < P1) then
  Movie.Description := Trim(Copy(HTML, 1, T - 1));

 //Rating
 P1 := Pos('<a href="/level/83/film/', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('<a href="/level/83/film/') - 1);
  P1 := Pos('">', HTML);
  Delete(HTMl, 1, P1 + 1);
  P2 := Pos('</a>', HTML);
  if P2 > 0 then begin
   try
    S := Tnt_WideStringReplace(Trim(Copy(HTML, 1, P2 - 1)), '.', DecimalSeparator, [rfReplaceAll]);
    Movie.Rating := StrToFloat(S);
   except
    Movie.Rating := 0;
   end;
   Movie.RatingName := RATING_STR;
  end;
 end;
end;

function NextMode(curMode : TWorkMode) : TWorkMode;
var
 I : Integer;
begin
 Result := wmFinished;

 for I := curMode + 1 to High(ExtraLinks) do
  if ExtraLinks[I] <> '' then begin
   Result := I;
   Break;
  end;
end;

(*
Results:
0 - Error
1 - OK (data extracted)
2 - Movie selection must be shown
*)
function Parse(URL : PCHAR; Data : PWideChar) : SHORT; stdcall;
var
 S : WideString;
begin
 S := Data;
 if (Pos('К сожалению, по вашему запросу ничего не найдено.', S) > 0) then begin
  SetLength(Selection, 0);
  Count := 0;
  Result := PLUGIN_RESPONSE_LIST;
  Exit;
 end;

 if (Pos('<title>Результаты поиска', S) > 0) then begin
  ParseSearchResults(S);
  Result := PLUGIN_RESPONSE_LIST;
 end else
 if (Pos('<!-- инфа о фильме -->', S) > 0) then begin
  ParseMovie(S);
  Movie.URL := URL;

  Result := PLUGIN_RESPONSE_OK;
 end else begin
  SetLength(Selection, 0);
  Count := 0;
  Result := PLUGIN_RESPONSE_LIST;
 end;
end;

function GetResultsCount : Integer; stdcall;
begin
 Result := Count;
end;

function GetSelection(const N : Integer; out Title, Title2 : PWideChar; out URL, Preview, Browse : PCHAR; out Year : Smallint) : BOOL; stdcall;
begin
 Result := False;
 if (N > High(Selection)) then
  Exit;

 Result := True;

 Title := PWideChar(Selection[N].Title);
 Title2 := PWideChar(Selection[N].Title2);
 URL := PChar(Selection[N].URL);
 Browse := PChar(Selection[N].Browse);
 Preview := PChar(Selection[N].Preview);
 Year := Selection[N].Year;
end;

function GetMovie(const N : Integer; var AMovieInfo : PWideChar) : BOOL; stdcall;
var
 D : Char;
begin
 D := DecimalSeparator;
 DecimalSeparator := '.';

 XML := CreateTag('xml', '', True, False);
 XML := XML + CreateTag('movie', '', True, False);
 if Movie.Title <> '' then
  XML := XML + CreateTag(XML_TITLE, HTMLToText(Movie.Title));
 if Movie.OrigTitle <> '' then
  XML := XML + CreateTag(XML_ORIGTITLE, HTMLToText(Movie.OrigTitle));
 if Movie.Aka <> '' then
  XML := XML + CreateTag(XML_AKA, HTMLToText(Movie.Aka));
 if Movie.Year <> 0 then
  XML := XML + CreateTag(XML_YEAR, IntToStr(Movie.Year), True, True, False);
 if Movie.Genre <> '' then
  XML := XML + CreateTag(XML_GENRE, HTMLToText(Movie.Genre));
 if Movie.Country <> '' then
  XML := XML + CreateTag(XML_COUNTRY, HTMLToText(Movie.Country));
 if Movie.Studio <> '' then
  XML := XML + CreateTag(XML_STUDIO, HTMLToText(Movie.Studio));
 if Movie.MPAA <> '' then
  XML := XML + CreateTag(XML_MPAA, HTMLToText(Movie.MPAA));
 if Movie.Rating <> 0 then begin
  XML := XML + CreateTag(XML_OTHERRATING, FloatToStrSQL(Movie.Rating), True, True, False);
  XML := XML + CreateTag(XML_OTHERNAME, RATING_STR);
 end;
 if Movie.Tags <> '' then
  XML := XML + CreateTag(XML_TAGS, HTMLToText(Movie.Tags));
 if Movie.Tagline <> '' then
  XML := XML + CreateTag(XML_TAGLINE, HTMLToText(Movie.Tagline));
 if Movie.Description <> '' then
  XML := XML + CreateTag(XML_DESCRIPTION, HTMLToText(Movie.Description));
 if Movie.Duration <> 0 then
  XML := XML + CreateTag(XML_DURATION, IntToStr(Movie.Duration * 60));
 if Movie.URL <> '' then
  XML := XML + CreateTag(XML_URL, Movie.URL);
 if Movie.Features <> '' then
  XML := XML + CreateTag(XML_FEATURES, HTMLToText(Movie.Features));

 XML := XML + Movie.Actors;
 XML := XML + Movie.Director;
 XML := XML + Movie.Producer;
 XML := XML + Movie.Scenario;
 XML := XML + Movie.Music;
 XML := XML + Movie.Links;
 XML := XML + Movie.Awards; //ALWAYS PLACE AWARDS AFTER CREDITS!!!
 XML := XML + Movie.Episodes;

 XML := XML + CreateTag('movie', '', False, True) + CreateTag('xml', '', False, True);

 AMovieInfo := PWideChar(XML);

 DecimalSeparator := D;

 Result := True;
end;  

function BeginWork : Integer; stdcall;
begin
 SetLength(Selection, 0);
 Count := 0;
 Mode := wmSearch;

 if WideFileExists(ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME) then
  Result := ERR_PLUG_OK
 else
  Result := ERR_PLUG_NEEDCONFIGURE;
end;

procedure EndWork; stdcall;
begin

end;


exports
  BeginWork,
  EndWork,
  GetName,
  GetDescription,
  GetType,
  GetVersionSupported,
  GetPluginVersion,
  Parse,
  GetResultsCount,
  GetSelection,
  GetMovie,
  GetURL,
  CanConfig,
  GetPluginLanguage,
  Configure,
  GetLoginInfo,
  GetReferrer,
  GetCodePage,
  CheckPVDVerion;

begin
end.
