library kinopoisk_people;

uses
  Windows,
  SysUtils,
  Classes,
  Controls,
  Forms,
  LoginConfigFormU in 'LoginConfigFormU.pas' {ConfigForm},
  TntSysUtils,
  ImportU,
  XMLGenU;

const
  PLUGIN_NAME   = 'Kinopoisk [RU]';
  PLUGIN_DESC_EN   = '[RU] Get Information about People from Kinopoisk.ru';
  PLUGIN_DESC_DE   = '[RU] Persondaten aus Kinopoisk.ru importieren';
  PLUGIN_DESC_RU   = '[RU]      Kinopoisk.ru';
  PLUGIN_TYPE   = PLUGIN_WEBPEOPLEIMPORT;
  PLUGIN_SUPVER = 9008000;
  PLUGIN_VER    = '0.2.1.2';
  PLUGIN_CONFIG_FILENAME = 'Plugins\kppconf.cfg';
  PLUGIN_ABOUT  = '%s' + #13#13 + 'by Nostradamus';
  PLUGIN_LANG   = LANG_RUSSIAN;
  PVD_SUPVER_MIN = 9008000;

  KINOPOISK_BASE = 'http://www.kinopoisk.ru';
  //KINOPOISK_SEARCH_STR = 'http://www.kinopoisk.ru/index.php?level=7&m_act[what]=actor&m_act[find]=%s';
  KINOPOISK_SEARCH_STR = 'http://www.kinopoisk.ru/index.php?kp_query=%s';
  KINOPOISK_LOGIN_URL  = 'http://www.kinopoisk.ru/level/30/';

  CODE_PAGE = 1251;

  STR_LOGIN_EN    = 'Login:';
  STR_PASSWORD_EN = 'Password:';
  STR_LOGIN_RU    = ':';
  STR_PASSWORD_RU = ':';
  STR_LOGIN_DE    = 'Benutzername:';
  STR_PASSWORD_DE = 'Passwort:';
  STR_SWITCH_NAMES_EN = 'Switch names';
  STR_SWITCH_NAMES_DE = 'Namen umtauschen';
  STR_SWITCH_NAMES_RU = '  ';

  wmFinished = 0;
  wmSearch = 1;
  wmMain = 2;
  wmAwards = 4;

type
  TWorkMode = wmFinished..wmAwards;

var
 Selection : array of TSelectionRec;
 Human : THumanRec;
 Count : Integer = 0;

 URLs : TURLList;
 Mode : TWorkMode;

 XML : WideString;

 doAwards : Boolean = True; //TODO: Option
 doSwitch : Boolean = False;

 PhotoLink, AwardsLink : String;

{$R *.res}

function GetName(const LangID : Integer): PWideChar; stdcall;
begin
 Result := PLUGIN_NAME;
end;

function GetDescription(const LangID : Integer) : PWideChar; stdcall;
begin
 case LangID of
  LANG_RUSSIAN : Result := PLUGIN_DESC_RU;
  LANG_GERMAN : Result := PLUGIN_DESC_DE;
  else
   Result := PLUGIN_DESC_EN;
 end;
end;

(*
Types:
0 - Unknown
1 - General Purpose
2 - Import from web
3 - Import genral
4 - Import Picture from Web
5 - Export
*)
function GetType : SHORT; stdcall;
begin
 Result := PLUGIN_TYPE;
end;

(*
Use 0 for all
*)
function GetVersionSupported : DWORD; stdcall;
begin
 Result := PLUGIN_SUPVER;
end;

function CheckPVDVerion(const V : Integer) : BOOL; stdcall;
begin
 Result := PVD_SUPVER_MIN <= V;
end;

function GetPluginVersion : PCHAR; stdcall;
begin
 Result := PLUGIN_VER;
end;

function GetPluginLanguage : Integer; stdcall;
begin
 Result := PLUGIN_LANG;
end;

function GetURL(const isEpisode : Boolean; const CurrentURL : PCHAR; out NewURL : PCHAR) : Integer; stdcall;
var
 I : Integer;
 Found : Boolean;
begin
 Result := WEB_URL_SEARCH;

 if PhotoLink = '' then begin
  Found := False;
  for I := 0 to ParseURLs(CurrentURL, URLs) - 1 do
   if Pos('kinopoisk.ru', LowerCase(URLs[I])) > 0 then begin
    Found := True;
    NewURL := PCHAR(URLs[I]);
    Result := WEB_URL_DIRECT;
   end;

  if not Found then begin
   Result := WEB_URL_SEARCH;

   NewURL := KINOPOISK_SEARCH_STR;
  end;
 end else begin
  NewURL := PCHAR(PhotoLink);
  Result := WEB_URL_DIRECT;
 end;
end;

function CanConfig : BOOL; stdcall;
begin
 Result := True;
end;

procedure GetConfigData(var Login, Password : String);
var
 F : TextFile;
 S : String;
begin
 if FileExists(ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME) then begin
   AssignFile(F, ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME);
  FileMode := 1;
  Reset(F);
  ReadLn(F, Login);
  ReadLn(F, Password);
  ReadLn(F, S);
  doSwitch := StrToBoolDef(S, False);
  CloseFile(F);
 end else begin
  Login := '';
  Password := '';
 end;
end;

function Configure(const LangID : Integer; AppHandle : HWND; CustomFields : PWideChar) : BOOL; stdcall;
var
 F : TextFile;
 ConfFrm : TConfigForm;
 L, P : String;
begin
 Result := False;

 GetConfigData(L, P);

 Application.Handle := AppHandle;
 ConfFrm := TConfigForm.Create(Application);
 try
  case LangID of
   LANG_RUSSIAN : begin
         ConfFrm.LoginLbl.Caption := STR_LOGIN_RU;
         ConfFrm.PassLbl.Caption := STR_PASSWORD_RU;
         ConfFrm.SwitchCheck.Caption := STR_SWITCH_NAMES_RU;
        end;
   LANG_GERMAN : begin
         ConfFrm.LoginLbl.Caption := STR_LOGIN_DE;
         ConfFrm.PassLbl.Caption := STR_PASSWORD_DE;
         ConfFrm.SwitchCheck.Caption := STR_SWITCH_NAMES_DE;
        end;
   else begin
    ConfFrm.LoginLbl.Caption := STR_LOGIN_EN;
    ConfFrm.PassLbl.Caption := STR_PASSWORD_EN;
    ConfFrm.SwitchCheck.Caption := STR_SWITCH_NAMES_EN;
   end;
  end;

  ConfFrm.LoginLbl.Left := ConfFrm.LoginEdit.Left - 6 - ConfFrm.LoginLbl.Width;
  ConfFrm.PassLbl.Left := ConfFrm.PassEdit.Left - 6 - ConfFrm.PassLbl.Width;


  ConfFrm.Caption := 'KinoPoisk.ru';

  ConfFrm.LoginEdit.Text := L;
  ConfFrm.PassEdit.Text := P;
  ConfFrm.SwitchCheck.Checked := doSwitch;
  if ConfFrm.ShowModal = mrOK then begin
   AssignFile(F, ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME);
   FileMode := 2;
   Rewrite(F);
   WriteLn(F, ConfFrm.LoginEdit.Text);
   WriteLn(F, ConfFrm.PassEdit.Text);
   WriteLn(F, BoolToStr(ConfFrm.SwitchCheck.Checked));
   CloseFile(F);

   Result := True;
  end;
 finally
  ConfFrm.Free;
 end;
end;

procedure GetLoginInfo(out ScriptURL : PCHAR; out Params : PCHAR); stdcall;
var
 L, P : String;
begin
 ScriptURL := KINOPOISK_LOGIN_URL;

 GetConfigData(L, P);

 if (L <> '') AND (P <> '') then
  Params := PCHAR(Format('shop_user[login]=%s&shop_user[pass]=%s', [L, P]))
 else
  Params := PCHAR('');
end;

function GetReferrer : PCHAR; stdcall;
begin
 Result := KINOPOISK_BASE;
end;

function GetCodePage : Cardinal; stdcall;
begin
 Result := CODE_PAGE;
end;

function GetAbout(const LangID : Integer) : PWideChar; stdcall;
begin
 case LangID of
  19 : Result := PWideChar(WideFormat(PLUGIN_ABOUT, [PLUGIN_DESC_RU]));
  07 : Result := PWideChar(WideFormat(PLUGIN_ABOUT, [PLUGIN_DESC_DE]));
  else
   Result := PWideChar(WideFormat(PLUGIN_ABOUT, [PLUGIN_DESC_EN]));
 end;
end;

function ReplaceTags(AText : WideString) : WideString;
const
 ltBad = -1;
 ltNone = 0;
 ltTitle = 1;
 ltName = 2;
 Tags : array [ltTitle..ltName] of WideString = ('<a href="/title/tt', '<a href="/name/nm');
var
 I, J : Integer;
 M : Integer;
 curValStart : Integer;
 Found, inTag : Boolean;
 Val : WideString;
begin
 Result := '';

 M := ltNone;
 inTag := False;
 curValStart := 0;
 for I := 1 to Length(AText) do begin
  if AText[I] = '<' then begin
   inTag := True;

   if M = ltNone then begin
    Found := False;
    for J := Low(Tags) to High(Tags) do
     if Copy(AText, I, Length(Tags[J])) = Tags[J] then begin
      M := J;
      curValStart := 0;
      Found := True;
      Break;
     end;

    if not Found then
     M := ltBad;
   end else
   if (M > ltNone) AND (Length(AText) > I) AND (AText[I + 1] = '/') then begin
    if curValStart > 0 then begin
     Val := Trim(Copy(AText, curValStart, I - curValStart));
     case M of
      ltTitle : Result := Result + '<link otitle="' + Val + '">' + Val + '</link>';
      ltName : Result := Result + '<link name="' + Val + '">' + Val + '</link>';
     end;
    end;

    curValStart := 0;
    M := ltNone;
   end;
  end else
  if AText[I] = '>' then begin
   inTag := False;
   if M > ltNone then
    curValStart := I + 1
   else
    M := ltNone; 
  end else
  if (not inTag) AND (M <= ltNone) then
   Result := Result + AText[I];
 end;
end;

procedure ParseSearchResults(var HTML : WideString);
  procedure ParseTitles(var S : WideString);
 var
  P, P1, P2, E : Integer;
  O : WideString;
 begin
  E := Pos('<!-- /  -->', S);
  if E < 1 then
   Exit;
  P := Pos('href="/level/4/people/', S);
  while (P > 0) AND (P < E) do begin
   Delete(S, 1, P + Length('href="/level/4/people/') - 1);
   P1 := Pos('/', S);
   if P1 > 0 then begin
    Inc(Count);
    SetLength(Selection, Length(Selection) + 1);
    Selection[Length(Selection)-1].URL := 'http://www.kinopoisk.ru/level/4/people/'  + Copy(S, 1, P1) + 'view_bio/ok/';
    Selection[Length(Selection)-1].Browse := Selection[Length(Selection)-1].URL;
    P1 := Pos('">', S);
    P2 := Pos('</a>', S);
    if (P1 > 0) AND (P2 > 0) then begin
     Selection[Length(Selection)-1].Title := Copy(S, P1 + 2, P2 - P1 - 2);
     Delete(S, 1, P2 + 3);

     //Original title
     P1 := Pos('<font color="#999999">... ', S);
     if (P1 > 0) AND (P1 < 15) then begin
      Delete(S, 1, P1 + Length('<font color="#999999">... ') - 1);
      P1 := Pos('</font>', S);
      if P1 > 0 then begin
       O := Copy(S, 1, P1 - 1);
       Delete(S, 1, P1 + Length('</font>') - 1);
      end;
     end;

     if O <> '' then
      Selection[Length(Selection)-1].Title := Selection[Length(Selection)-1].Title + ' / ' + O;

     E := Pos('<!-- /  -->', S);
    end;
   end;
   P := Pos('href="/level/4/people/', S);
  end;
 end;
var
 P, B, L : Integer;
begin
 Count := 0;
 SetLength(Selection, 0);

 B := High(Integer);
 L := 0;

 P := Pos('<b>   :</b>', HTML);
 if P < B then begin
  B := P;
  L := Length('<b>   :</b>');
 end;

 P := Pos('<b> :</b>', HTML);
 if P < B then begin
  B := P;
  L := Length('<b> :</b>');
 end;

 if B < High(Integer) then begin
  Delete(HTML, 1, B + L - 1);
  ParseTitles(HTML);
 end;
end;

function FindPicture(var HTML : WideString) : Boolean;
var
 P1, P2: Integer;
begin
 Result := False;

 P1 := Pos('/images/actor/', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 - 1);
  P2 := Pos('''', HTML);
  if P2 > 0 then begin
   Result := True;
   PhotoLink := 'http://www.kinopoisk.ru' + Copy(HTML, 1, P2 - 1);
  end;
 end;
end;

procedure ParseHuman(var HTML : WideString);
 function KinopoiskDateStrToDate(Str1, Str2 : WideString) : WideString;
 const
   Months : array [1..12] of WideString = ('', '', '', '', '', '', '', '', '', '', '', '');
 var
  P, I : Integer;
  Y, M, D : Word;
 begin
  Result := '';

  P := Pos(' ', Str1);
  try
   D := StrToInt(Trim(Copy(Str1, 1, P - 1)));
  except
   Exit;
  end;
  Delete(Str1, 1, P);
  Str1 := Trim(Str1);

  M := 0;
  for I := 1 to 12 do
   if WideCompareText(Months[I], Str1) = 0 then begin
    M := I;
    Break;
   end;

  if M = 0 then
   Exit;

  try
   Y := StrToInt(Trim(Str2));
  except
   Exit;
  end;

  Result := WideFormat('%d-%d-%d', [Y, M, D]);
 end;
function GetCareerType(AVal : WideString) : Smallint;
const
  TableDefs : array [0..4] of Smallint = (
   0, 1, 4, 3, 2
  );
  FilmDefs : array [0..4, 0..1] of WideString = (
   ('', ''),
   ('c', ''),
   ('', ''),
   ('', ''),
   ('', '')
  );
var
 I, J : Integer;
begin
 Result := -1;

 AVal := Trim(AVal);

 for I := Low(FilmDefs) to High(FilmDefs) do
  for J := Low(FilmDefs[I]) to High(FilmDefs[I]) do
   if CompareText(FilmDefs[I, J], AVal) = 0 then begin
    Result := TableDefs[I];
    Exit;
   end;
end;
var
 P1, P2, T, E : Integer;
 tmpStr1, tmpStr2, title, origtitle, role, year, url : WideString;
 CT : Integer;

 Dummy : String;
begin
 Count := 1;

 GetConfigData(Dummy, Dummy);
 tmpStr1 := '';

 //Translated name
 P1 := Pos('class="moviename-big">', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('class="moviename-big">') - 1);

  P1 := Pos('<STRONG>', HTML);
  if P1 > 0 then begin
   Delete(HTML, 1, P1 + Length('<STRONG>') - 1);
   P1 := Pos('</STRONG>', HTML);
   if (P1 > 0) then begin
    if not doSwitch then
     Human.TransName := Copy(HTML, 1, P1 - 1)
    else
     Human.Name := Copy(HTML, 1, P1 - 1);
    Delete(HTML, 1, P1 + Length('</STRONG>') - 1);
   end;
  end;
 end;

 FindPicture(HTML);

 //Name
 P1 := Pos(' (.)</td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length(' (.)</td>') - 1);

  P1 := Pos('<td class="desc-data"', HTML);
  if P1 > 0 then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data"') - 1);
   P1 := Pos('">', HTML);
   Delete(HTML, 1, P1 + 1);
   P1 := Pos('</td>', HTML);
   if (P1 > 0) then begin
    if not doSwitch then
     Human.Name := Copy(HTML, 1, P1 - 1)
    else
     Human.TransName := Copy(HTML, 1, P1 - 1);
    Delete(HTML, 1, P1 + Length('</td>') - 1);
   end;
  end;
 end;

 //Birthday
 P1 := Pos(' </td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length(' </td>') - 1);
  P1 := Pos('<td class="desc-data"', HTML);
  if P1 > 0 then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data"') - 1);

   P1 := Pos('href="/level/10/m_act%5Bbirthday', HTML);
   if P1 > 0 then begin
    Delete(HTML, 1, P1 + Length('href="/level/10/m_act%5Bbirthday') - 1);
    P1 := Pos('">', HTML);
    Delete(HTML, 1, P1 + 1);
    P1 := Pos('</a>', HTML);
    if (P1 > 0) then begin
     tmpStr1 := Copy(HTML, 1, P1 - 1);
     Delete(HTML, 1, P1 + Length('</a>') - 1);
    end;
   end;
   P1 := Pos('/level/10/m_act%5Bbirthday', HTML);
   if P1 > 0 then begin
    Delete(HTML, 1, P1 + Length('/level/10/m_act%5Bbirthday') - 1);
    P1 := Pos('">', HTML);
    Delete(HTML, 1, P1 + 1);
    P1 := Pos('</a>', HTML);
    if (P1 > 0) then begin
     tmpStr2 := Copy(HTML, 1, P1 - 1);
     Delete(HTML, 1, P1 + Length('</a>') - 1);
    end;
   end;
   Human.Birthday := KinopoiskDateStrToDate(tmpStr1, tmpStr2);
  end;
 end;

 //Birthplace
 P1 := Pos(' </td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length(' </td>') - 1);

  P1 := Pos('<td class="desc-data', HTML);
  if P1 > 0 then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data') - 1);
   P1 := Pos('">', HTML);
   Delete(HTML, 1, P1 + 1);
   P1 := Pos('</td>', HTML);
   if (P1 > 0) then begin
    Human.Birthplace := Trim(Copy(HTML, 1, P1 - 1));
    if Human.Birthplace = '-' then
     Human.Birthplace := '';
    Delete(HTML, 1, P1 + Length('</td>') - 1);
   end;
  end;
 end;

 P1 := Pos(' </td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length(' </td>') - 1);
  P1 := Pos('<td class="desc-data">', HTML);
  if P1 > 0 then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data">') - 1);

   P1 := Pos('</td>', HTML);
   P2 := Pos('<font', HTML);
   if (P2 > 0) AND (P2 < P1) then
    P1 := P2;
   if P1 > 0 then begin
    tmpStr1 := Copy(HTML, 1, P1 - 1);
    Delete(HTML, 1, P1 + Length('</td>') - 1);
    P1 := Pos(',', tmpStr1);
    if (P1 > 0) then begin
     tmpStr2 := Copy(tmpStr1, P1 + 1, Length(tmpStr1) - P1);
     Delete(tmpStr1, P1, Length(tmpStr1) - P1 + 1);
     Human.DeathDate := KinopoiskDateStrToDate(tmpStr1, tmpStr2);
    end;
   end;
  end;
 end;

 //Genre(s)
 P1 := Pos('</td>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('</td>') - 1);
  P1 := Pos('<td class="desc-data">', HTML);
  if P1 > 0 then begin
   Delete(HTML, 1, P1 + Length('<td class="desc-data">') - 1);
   T := Pos(' href="/level/10/', HTML);
   E := Pos('</td>', HTML);
   while (T > 0) AND (T < E) do begin
    Delete(HTML, 1, T + Length(' href="/level/10/') - 1);
    P1 := Pos('">', HTML);
    P2 := Pos('</a>', HTML);

    if (P1 > 0) AND (P2 > P1) then begin
     if Human.Genre <> '' then
      Human.Genre := Human.Genre + ', ';
     Human.Genre := Human.Genre + Copy(HTML, P1 + 2, P2 - (P1 + 2));

     Delete(HTML, 1, P2 + Length('</a>') - 1);
    end;

    T := Pos(' href="/level/10/', HTML);
    E := Pos('</td>', HTML);
   end;

   Delete(HTML, 1, E + Length('</td>') - 1);
  end;
 end;

 //Bio
 P1 := Pos('<H2 class="chapter"></H2>', HTML);
 if P1 > 0 then begin
  Delete(HTML, 1, P1 + Length('<H2 class="chapter"></H2>') - 1);

  P1 := Pos('class=news>', HTML);
  Delete(HTML, 1, P1 + Length('class=news>') - 1);
  P1 := Pos('</td>', HTML);
  T := Pos('<a href=', HTML);

  if (P1 < T) then
   T := P1;

  Human.Bio := Trim(Copy(HTML, 1, T - 1));
 end;

 //Filmography
 Human.Films := CreateTag(XML_FILMOGRAPHY, '', True, False);
 try
  CT := 0;
  P1 := Pos('<H2 class="chapter">', HTML);
  while (P1 > 0) AND (Pos('<!--   -->', HTML) > P1) do begin
   Delete(HTML, 1, P1 + Length('<H2 class="chapter">') - 1);

   P1 := Pos('</H2>', HTML);
   if P1 > 0 then
    CT := GetCareerType(Trim(Copy(HTML, 1, P1 - 1)));

   P1 := Pos('<H2 class="chapter">', HTML);
   P2 := Pos('href="/level/1/film/', HTML);
   while (P2 > 0) AND ((P2 < P1) OR (P1 = 0)) AND (Pos('<!--   -->', HTML) > P2) AND (CT > -1) do begin
    title := '';
    origtitle := '';
    role := '';
    year := '';
    url := '';

    Delete(HTML, 1, P2 + Length('href="/level/1/film/') - 1);

    P1 := Pos('">', HTML);
    url := KINOPOISK_BASE + '/level/1/film/' + Copy(HTML, 1, P1 - 1);
    Delete(HTML, 1, P1 + 1);

    P1 := Pos('</a>', HTML);
    title := Copy(HTML, 1, P1 - 1);
    Delete(HTML, 1, P1 + 3);

    P1 := LastPos('&nbsp;', title);
    if (P1 > 0) AND (P1 < P2) then begin
     year := Copy(title, P1 + 7, 4);

     Delete(title, P1, Length(title) - P1 + 1);
    end;

    title := HTMLToText(title);
    title := Tnt_WideStringReplace(title, '()', '', [rfReplaceAll]);
    title := Tnt_WideStringReplace(title, '()', '', [rfReplaceAll]);
    title := Tnt_WideStringReplace(title, '()', '', [rfReplaceAll]);
    title := Trim(title);

    P1 := Pos('<span style="color:#999">', HTML);
    if P1 > 0 then begin
     Delete(HTML, 1, P1 + Length('<span style="color:#999">') - 1);
     P1 := Pos('&nbsp;', HTML);
     P2 := Pos('</span>', HTML);
     if (P2 > 0) AND (P2 < P1) then begin
      origtitle := Trim(HTMLToText(Copy(HTML, 1, P2 - 1)));
      Delete(HTML, 1, P2 + 6);
     end else begin
      origtitle := Trim(HTMLToText(Copy(HTML, 1, P1 - 1)));
      Delete(HTML, 1, P1 + 5);

      P1 := Pos('</', HTML);
      P2 := Pos('&nbsp;', HTML);
      if (P2 > 0) AND (P2 < P1) then begin
       Delete(HTML, 1, P2 + 5);
       P1 := Pos('</', HTML);

       role := Trim(Tnt_WideStringReplace(HTMLToText(Copy(HTML, 1, P1 - 1)), '...', '', [rfReplaceall]));
      end;
     end;
    end;

    {P1 := Pos('/level/10/m_act', HTML);
    P2 := Pos('href="/level/1/film/', HTML);
    if (P1 > 0) AND ((P2 < 1) OR (P1 < P2)) then begin
     Delete(HTML, 1, P1 + Length('/level/10/m_act') - 1);
     P1 := Pos('>', HTML);
     Delete(HTML, 1, P1);
     P1 := Pos('</a>', HTML);
     year := Trim(Copy(HTML, 1, P1 - 1));
     if Length(year) > 4 then
      Delete(year, 5, Length(year) - 4);
    end;

    P1 := Pos('&nbsp;...&nbsp;', HTML);
    P2 := Pos('href="/level/1/film/', HTML);
    if (P1 > 0) AND ((P2 < 1) OR (P1 < P2)) then begin
     Delete(HTML, 1, P1 + Length('&nbsp;...&nbsp;') - 1);
     P1 := Pos('</span>', HTML);
     role := HTMLToText(Trim(Copy(HTML, 1, P1 - 1)));
    end;

    P1 := Pos('<br>', HTML);
    Delete(HTMl, 1, P1 + 3);
    P1 := Pos('<font color="#aaaaaa">', HTML);
    if P1 > 0 then begin
     Delete(HTML, 1, P1 + Length('<font color="#aaaaaa">') - 1);
     P1 := Pos('</font>', HTML);
     origtitle := Trim(HTMLToText(Copy(HTML, 1, P1 - 1)));
    end;}
    Human.Films := Human.Films + CreateTag('film', CreateTag(XML_ORIGTITLE, OrigTitle) + CreateTag(XML_TITLE, Title) + CreateTag(XML_URL, URL) +
                                 CreateTag('role', Role) + CreateTag(XML_YEAR, Year) + CreateTag('careertype', IntToStr(CT)), True, True, False);

    P1 := Pos('<H2 class="chapter">', HTML);
    P2 := Pos('href="/level/1/film/', HTML);
   end;

   P1 := Pos('<H2 class="chapter">', HTML);
  end;
 finally
  Human.Films := Human.Films + CreateTag(XML_FILMOGRAPHY, '', False, True);
 end;
end;


function Parse(URL : PCHAR; Data : PWideChar) : SHORT; stdcall;
var
 S : WideString;
begin
 PhotoLink := '';

 S := Data;
 if (Pos(' ,      .', S) > 0) then begin
  SetLength(Selection, 0);
  Count := 0;
  Result := PLUGIN_RESPONSE_LIST;
  Exit;
 end;

 if Pos('<!--    -->', S) > 0 then begin
  ClearHuman(Human);
  Human.URL := URL;
  ParseHuman(S);
  if PhotoLink <> '' then
   Result := PLUGIN_RESPONSE_DOWNLOAD
  else
   Result := PLUGIN_RESPONSE_OK;
 end else
 if Pos('<title> ', S) > 0 then begin
  ParseSearchResults(S);
  Result := PLUGIN_RESPONSE_LIST;
 end else
  Result := PLUGIN_RESPONSE_ERROR;
end;

function GetResultsCount : Integer; stdcall;
begin
 Result := Count;
end;

function GetSelection(const N : Integer; out Title, Title2 : PWideChar; out URL, Preview, Browse : PCHAR; out Year : Smallint) : BOOL; stdcall;
begin
 Result := False;
 if (N > High(Selection)) then
  Exit;

 Result := True;

 Title := PWideChar(Selection[N].Title);
 Title2 := PWideChar(Selection[N].Title2);
 URL := PChar(Selection[N].URL);
 Browse := PChar(Selection[N].Browse);
 Preview := PChar(Selection[N].Preview);
 Year := Selection[N].Year;
end;

function GetHuman(const N : Integer; var AHumanInfo : PWideChar) : BOOL; stdcall;
var
 D : Char;
begin
 D := DecimalSeparator;
 DecimalSeparator := '.';

 XML := CreateTag('xml', '', True, False);
 XML := XML + CreateTag('person', '', True, False);

 if Human.Name <> '' then
  XML := XML + CreateTag(XML_NAME, HTMLToText(Human.Name));
 if Human.TransName <> '' then
  XML := XML + CreateTag(XML_TRANSNAME, HTMLToText(Human.TransName));
 if Human.AltNames <> '' then
  XML := XML + CreateTag(XML_ALTNAMES, HTMLToText(Human.AltNames));
 if Human.Birthday <> '' then
  XML := XML + CreateTag(XML_BIRTHDAY, Human.Birthday);
 if Human.Birthplace <> '' then
  XML := XML + CreateTag(XML_BIRTHPLACE, HTMLToText(Human.Birthplace));
 if Human.DeathDate <> '' then
  XML := XML + CreateTag(XML_DEATH, HTMLToText(Human.DeathDate));
 if Human.Genre <> '' then
  XML := XML + CreateTag(XML_GENRE, HTMLToText(Human.Genre));
 if Human.URL <> '' then
  XML := XML + CreateTag(XML_URL, HTMLToText(Human.URL));
 if Human.Bio <> '' then
  XML := XML + CreateTag(XML_BIO, HTMLToText(Human.Bio));

 XML := XML + Human.Awards;
 XML := XML + Human.Films; 

 XML := XML + CreateTag('person', '', False, True) + CreateTag('xml', '', False, True);

 AHumanInfo := PWideChar(XML);

 DecimalSeparator := D;

 {if PhotoLink <> '' then
  Result := PLUGIN_RESPONSE_OKPHOTO
 else}
  Result := True;
end;

function BeginWork : Integer; stdcall;
begin
 SetLength(Selection, 0);
 Count := 0;
 PhotoLink := '';
 AwardsLink := '';
 Mode := wmSearch;

 if WideFileExists(ExtractFilePath(ParamStr(0)) + PLUGIN_CONFIG_FILENAME) then
  Result := ERR_PLUG_OK
 else
  Result := ERR_PLUG_NEEDCONFIGURE; 
end;

procedure EndWork; stdcall;
begin

end;


exports
  BeginWork,
  EndWork,
  GetName,
  GetDescription,
  GetType,
  GetVersionSupported,
  GetPluginVersion,
  Parse,
  GetResultsCount,
  GetSelection,
  GetHuman,
  GetURL,
  CanConfig,
  Configure,
  GetAbout,
  GetPluginLanguage,
  GetLoginInfo,
  GetReferrer,
  GetCodePage,
  CheckPVDVerion;

begin
end.
